# SPDX-FileCopyrightText: 2018-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from collections import OrderedDict

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import HostsOrdering


class GmpModifyTaskTestMixin:
    def test_modify_task(self):
        self.gmp.modify_task("t1")

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"/>'
        )

    def test_modify_task_missing_task_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_task(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_task("")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_task(task_id="")

    def test_modify_task_with_name(self):
        self.gmp.modify_task(task_id="t1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><name>foo</name></modify_task>'
        )

    def test_modify_task_with_config_id(self):
        self.gmp.modify_task(task_id="t1", config_id="c1")

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><config id="c1"/></modify_task>'
        )

    def test_modify_task_with_target_id(self):
        self.gmp.modify_task(task_id="t1", target_id="t1")

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><target id="t1"/></modify_task>'
        )

    def test_modify_task_with_scanner_id(self):
        self.gmp.modify_task(task_id="t1", scanner_id="s1")

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><scanner id="s1"/></modify_task>'
        )

    def test_modify_task_with_agent_group_id(self):
        self.gmp.modify_task(task_id="t1", agent_group_id="ag1")

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><agent_group id="ag1"/></modify_task>'
        )

    def test_modify_task_with_oci_image_target_id(self):
        self.gmp.modify_task(task_id="t1", oci_image_target_id="it1")

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><oci_image_target id="it1"/></modify_task>'
        )

    def test_modify_task_with_target_and_agent_group(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_task(
                task_id="t1", target_id="t1", agent_group_id="ag1"
            )

    def test_modify_task_with_target_and_oci_image_target(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_task(
                task_id="t1", target_id="t1", oci_image_target_id="it1"
            )

    def test_modify_task_with_agent_group_and_oci_image_target(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_task(
                task_id="t1", agent_group_id="ag1", oci_image_target_id="it1"
            )

    def test_modify_task_with_schedule_id(self):
        self.gmp.modify_task(task_id="t1", schedule_id="s1")

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><schedule id="s1"/></modify_task>'
        )

    def test_modify_task_with_comment(self):
        self.gmp.modify_task(task_id="t1", comment="bar")

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><comment>bar</comment></modify_task>'
        )

    def test_modify_task_with_alerts_ids(self):
        self.gmp.modify_task(task_id="t1", alert_ids=["a1", "a2", "a3"])

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1">'
            b'<alert id="a1"/>'
            b'<alert id="a2"/>'
            b'<alert id="a3"/>'
            b"</modify_task>"
        )

    def test_modify_task_with_empty_alert_ids(self):
        self.gmp.modify_task(task_id="t1", alert_ids=[])

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><alert id="0"/></modify_task>'
        )

    def test_modify_task_with_alterable(self):
        self.gmp.modify_task(task_id="t1", alterable=True)

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><alterable>1</alterable></modify_task>'
        )

        self.gmp.modify_task(task_id="t1", alterable=False)

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><alterable>0</alterable></modify_task>'
        )

    def test_modify_task_with_hosts_ordering(self):
        self.gmp.modify_task(task_id="t1", hosts_ordering=HostsOrdering.REVERSE)

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1">'
            b"<hosts_ordering>reverse</hosts_ordering>"
            b"</modify_task>"
        )

    def test_modify_task_invalid_hosts_ordering(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_task(task_id="t1", hosts_ordering="foo")

    def test_modify_task_with_schedule(self):
        self.gmp.modify_task(task_id="t1", schedule_id="s1")

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1"><schedule id="s1"/></modify_task>'
        )

    def test_modify_task_with_schedule_periods(self):
        self.gmp.modify_task(task_id="t1", schedule_periods=0)

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1">'
            b"<schedule_periods>0</schedule_periods>"
            b"</modify_task>"
        )

        self.gmp.modify_task(task_id="t1", schedule_periods=5)

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1">'
            b"<schedule_periods>5</schedule_periods>"
            b"</modify_task>"
        )

    def test_modify_task_invalid_schedule_periods(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_task(task_id="t1", schedule_periods="foo")

        with self.assertRaises(InvalidArgument):
            self.gmp.modify_task(task_id="t1", schedule_periods=-1)

    def test_modify_task_with_observers(self):
        self.gmp.modify_task(task_id="t1", observers=["u1", "u2"])

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1">'
            b"<observers>u1,u2</observers>"
            b"</modify_task>"
        )

    def test_modify_task_with_preferences(self):
        self.gmp.modify_task(
            task_id="t1",
            preferences=OrderedDict([("foo", "bar"), ("lorem", "ipsum")]),
        )

        self.connection.send.has_been_called_with(
            b'<modify_task task_id="t1">'
            b"<preferences>"
            b"<preference>"
            b"<scanner_name>foo</scanner_name>"
            b"<value>bar</value>"
            b"</preference>"
            b"<preference>"
            b"<scanner_name>lorem</scanner_name>"
            b"<value>ipsum</value>"
            b"</preference>"
            b"</preferences>"
            b"</modify_task>"
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.protocols.gmp.requests.v224 import Authentication


class AuthenticationTestCase(unittest.TestCase):
    def test_authenticate(self) -> None:
        request = Authentication.authenticate("admin", "admin")

        self.assertIsInstance(request, Request)
        self.assertEqual(
            bytes(request),
            b"<authenticate><credentials><username>admin</username>"
            b"<password>admin</password></credentials></authenticate>",
        )

    def test_authenticate_missing_username(self) -> None:
        with self.assertRaises(RequiredArgument):
            Authentication.authenticate(None, "foo")  # type: ignore

        with self.assertRaises(RequiredArgument):
            Authentication.authenticate("", "foo")

    def test_authenticate_missing_password(self) -> None:
        with self.assertRaises(RequiredArgument):
            Authentication.authenticate("bar", None)  # type: ignore

        with self.assertRaises(RequiredArgument):
            Authentication.authenticate("bar", "")

    def test_describe_auth(self) -> None:
        request = Authentication.describe_auth()

        self.assertIsInstance(request, Request)
        self.assertEqual(bytes(request), b"<describe_auth/>")

    def test_modify_auth(self) -> None:
        request = Authentication.modify_auth(
            "foo",
            dict([("foo", "bar"), ("lorem", "ipsum")]),  # noqa: C406
        )

        self.assertIsInstance(request, Request)
        self.assertEqual(
            bytes(request),
            b"<modify_auth>"
            b'<group name="foo">'
            b"<auth_conf_setting>"
            b"<key>foo</key>"
            b"<value>bar</value>"
            b"</auth_conf_setting>"
            b"<auth_conf_setting>"
            b"<key>lorem</key>"
            b"<value>ipsum</value>"
            b"</auth_conf_setting>"
            b"</group>"
            b"</modify_auth>",
        )

    def test_modify_auth_missing_group_name(self) -> None:
        with self.assertRaises(RequiredArgument):
            Authentication.modify_auth(
                group_name=None,
                auth_conf_settings={"foo": "bar"},  # type: ignore
            )

        with self.assertRaises(RequiredArgument):
            Authentication.modify_auth(
                group_name="", auth_conf_settings={"foo": "bar"}
            )

        with self.assertRaises(RequiredArgument):
            Authentication.modify_auth("", auth_conf_settings={"foo": "bar"})

    def test_modify_auth_auth_conf_settings(self) -> None:
        with self.assertRaises(RequiredArgument):
            Authentication.modify_auth(
                group_name="foo",
                auth_conf_settings=None,  # type: ignore
            )

        with self.assertRaises(RequiredArgument):
            Authentication.modify_auth(group_name="foo", auth_conf_settings="")  # type: ignore

        with self.assertRaises(RequiredArgument):
            Authentication.modify_auth("foo", "")  # type: ignore

        with self.assertRaises(RequiredArgument):
            Authentication.modify_auth("foo", {})



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetPortListsTestMixin:
    def test_get_port_lists(self):
        self.gmp.get_port_lists()

        self.connection.send.has_been_called_with(b"<get_port_lists/>")

    def test_get_port_lists_with_filter_string(self):
        self.gmp.get_port_lists(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_port_lists filter="foo=bar"/>'
        )

    def test_get_port_lists_with_filter_id(self):
        self.gmp.get_port_lists(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_port_lists filt_id="f1"/>'
        )

    def test_get_port_lists_with_trash(self):
        self.gmp.get_port_lists(trash=True)

        self.connection.send.has_been_called_with(
            b'<get_port_lists trash="1"/>'
        )

        self.gmp.get_port_lists(trash=False)

        self.connection.send.has_been_called_with(
            b'<get_port_lists trash="0"/>'
        )

    def test_get_port_lists_with_details(self):
        self.gmp.get_port_lists(details=True)

        self.connection.send.has_been_called_with(
            b'<get_port_lists details="1"/>'
        )

        self.gmp.get_port_lists(details=False)

        self.connection.send.has_been_called_with(
            b'<get_port_lists details="0"/>'
        )

    def test_get_port_lists_with_targets(self):
        self.gmp.get_port_lists(targets=True)

        self.connection.send.has_been_called_with(
            b'<get_port_lists targets="1"/>'
        )

        self.gmp.get_port_lists(targets=False)

        self.connection.send.has_been_called_with(
            b'<get_port_lists targets="0"/>'
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .versions import GmpGetProtocolVersionTestCase, GmpGetVersionTestCase


class Gmpv224GetVersionCommandTestCase(GmpGetVersionTestCase, Gmpv224TestCase):
    pass


class Gmpv224GmpGetProtocolVersionTestCase(
    GmpGetProtocolVersionTestCase, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Main module of python-gvm.
"""

from .__version__ import __version__


def get_version() -> str:
    """Returns the version of python-gvm as a string in
    `PEP440 <https://www.python.org/dev/peps/pep-0440>`_ compliant format.

    Returns:
        Current version of python-gvm
    """
    return __version__



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.utils import to_bool


class TestToBool(unittest.TestCase):
    def test_to_bool(self):
        true = to_bool(True)
        self.assertEqual("1", true)

        false = to_bool(False)
        self.assertEqual("0", false)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .hosts import (
    GmpCreateHostTestMixin,
    GmpDeleteHostTestMixin,
    GmpGetHostsTestMixin,
    GmpGetHostTestMixin,
    GmpModifyHostTestMixin,
)


class Gmpv224CreateHostTestCase(GmpCreateHostTestMixin, Gmpv224TestCase):
    pass


class Gmpv224DeleteHostTestCase(GmpDeleteHostTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetHostTestCase(GmpGetHostTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetHostsTestCase(GmpGetHostsTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyHostTestCase(GmpModifyHostTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v226 import CredentialType


class GetCredentialTypeFromStringTestCase(unittest.TestCase):
    def test_invalid_type(self):
        with self.assertRaises(InvalidArgument):
            CredentialType.from_string("foo")

    def test_none_or_empty_type(self):
        ct = CredentialType.from_string(None)
        self.assertIsNone(ct)
        ct = CredentialType.from_string("")
        self.assertIsNone(ct)

    def test_client_certificate(self):
        ct = CredentialType.from_string("client_certificate")
        self.assertEqual(ct, CredentialType.CLIENT_CERTIFICATE)

    def test_snmp(self):
        ct = CredentialType.from_string("snmp")
        self.assertEqual(ct, CredentialType.SNMP)

    def test_username_password(self):
        ct = CredentialType.from_string("username_password")
        self.assertEqual(ct, CredentialType.USERNAME_PASSWORD)

    def test_username_ssh_key(self):
        ct = CredentialType.from_string("username_ssh_key")
        self.assertEqual(ct, CredentialType.USERNAME_SSH_KEY)

    def test_smime_certificate(self):
        ct = CredentialType.from_string("smime_certificate")
        self.assertEqual(ct, CredentialType.SMIME_CERTIFICATE)

    def test_pgp_encryption_key(self):
        ct = CredentialType.from_string("pgp_encryption_key")
        self.assertEqual(ct, CredentialType.PGP_ENCRYPTION_KEY)

    def test_password_only(self):
        ct = CredentialType.from_string("password_only")
        self.assertEqual(ct, CredentialType.PASSWORD_ONLY)



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_get_vulnerabilities import GmpGetVulnerabilitiesTestMixin
from .test_get_vulnerability import GmpGetVulnerabilityTestMixin

__all__ = ("GmpGetVulnerabilitiesTestMixin", "GmpGetVulnerabilityTestMixin")



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import (
    SnmpAuthAlgorithm,
    SnmpPrivacyAlgorithm,
)


class GmpModifyCredentialTestMixin:
    def test_modify_credential(self):
        self.gmp.modify_credential(credential_id="c1")

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1"/>'
        )

    def test_modify_credential_missing_credential_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_credential(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_credential("")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_credential(credential_id="")

    def test_modify_credential_with_name(self):
        self.gmp.modify_credential(credential_id="c1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<name>foo</name>"
            b"</modify_credential>"
        )

    def test_modify_credential_with_comment(self):
        self.gmp.modify_credential(credential_id="c1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<comment>foo</comment>"
            b"</modify_credential>"
        )

    def test_modify_credential_with_certificate(self):
        self.gmp.modify_credential(credential_id="c1", certificate="abcdef")

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<certificate>abcdef</certificate>"
            b"</modify_credential>"
        )

    def test_modify_credential_with_private_key_and_key_phrase(self):
        self.gmp.modify_credential(
            credential_id="c1", private_key="123456", key_phrase="foo"
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<key>"
            b"<phrase>foo</phrase>"
            b"<private>123456</private>"
            b"</key>"
            b"</modify_credential>"
        )

    def test_modify_credential_missing_private_key(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_credential(credential_id="c1", key_phrase="foo")

    def test_modify_credential_missing_key_phrase(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_credential(credential_id="c1", private_key="123456")

    def test_modify_credential_with_allow_insecure(self):
        self.gmp.modify_credential(credential_id="c1", allow_insecure=True)

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<allow_insecure>1</allow_insecure>"
            b"</modify_credential>"
        )

        self.gmp.modify_credential(credential_id="c1", allow_insecure=False)

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<allow_insecure>0</allow_insecure>"
            b"</modify_credential>"
        )

    def test_modify_credential_with_login(self):
        self.gmp.modify_credential(credential_id="c1", login="foo")

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<login>foo</login>"
            b"</modify_credential>"
        )

    def test_modify_credential_with_password(self):
        self.gmp.modify_credential(credential_id="c1", password="foo")

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<password>foo</password>"
            b"</modify_credential>"
        )

    def test_modify_credential_with_auth_algorithm(self):
        self.gmp.modify_credential(
            credential_id="c1", auth_algorithm=SnmpAuthAlgorithm.MD5
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<auth_algorithm>md5</auth_algorithm>"
            b"</modify_credential>"
        )

        self.gmp.modify_credential(
            credential_id="c1", auth_algorithm=SnmpAuthAlgorithm.SHA1
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<auth_algorithm>sha1</auth_algorithm>"
            b"</modify_credential>"
        )

    def test_modify_credential_invalid_auth_algorithm(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_credential(credential_id="c1", auth_algorithm="foo")

    def test_modify_credential_with_community(self):
        self.gmp.modify_credential(credential_id="c1", community="foo")

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<community>foo</community>"
            b"</modify_credential>"
        )

    def test_modify_credential_with_privacy_algorithm(self):
        self.gmp.modify_credential(
            credential_id="c1", privacy_algorithm=SnmpPrivacyAlgorithm.AES
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<privacy>"
            b"<algorithm>aes</algorithm>"
            b"</privacy>"
            b"</modify_credential>"
        )

        self.gmp.modify_credential(
            credential_id="c1", privacy_algorithm=SnmpPrivacyAlgorithm.DES
        )

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<privacy>"
            b"<algorithm>des</algorithm>"
            b"</privacy>"
            b"</modify_credential>"
        )

    def test_modify_credential_invalid_privacy_algorithm(self):
        with self.assertRaises(ValueError):
            self.gmp.modify_credential(credential_id="c1", privacy_algorithm="")

        with self.assertRaises(InvalidArgument):
            self.gmp.modify_credential(
                credential_id="c1", privacy_algorithm="foo"
            )

    def test_modify_credential_with_privacy_password(self):
        self.gmp.modify_credential(credential_id="c1", privacy_password="foo")

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<privacy>"
            b"<password>foo</password>"
            b"</privacy>"
            b"</modify_credential>"
        )

    def test_modify_credential_with_public_key(self):
        self.gmp.modify_credential(credential_id="c1", public_key="foo")

        self.connection.send.has_been_called_with(
            b'<modify_credential credential_id="c1">'
            b"<key>"
            b"<public>foo</public>"
            b"</key>"
            b"</modify_credential>"
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.groups import (
    GmpCloneGroupTestMixin,
    GmpCreateGroupTestMixin,
    GmpDeleteGroupTestMixin,
    GmpGetGroupsTestMixin,
    GmpGetGroupTestMixin,
    GmpModifyGroupTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225DeleteGroupTestCase(GmpDeleteGroupTestMixin, GMPTestCase):
    pass


class Gmpv225GetGroupTestCase(GmpGetGroupTestMixin, GMPTestCase):
    pass


class Gmpv225GetGroupsTestCase(GmpGetGroupsTestMixin, GMPTestCase):
    pass


class Gmpv225CloneGroupTestCase(GmpCloneGroupTestMixin, GMPTestCase):
    pass


class Gmpv225CreateGroupTestCase(GmpCreateGroupTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyGroupTestCase(GmpModifyGroupTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .tls_certificates import (
    GmpCloneTLSCertificateTestMixin,
    GmpCreateTLSCertificateTestMixin,
    GmpDeleteTLSCertificateTestMixin,
    GmpGetTLSCertificatesTestMixin,
    GmpGetTLSCertificateTestMixin,
    GmpModifyTLSCertificateTestMixin,
)


class Gmpv224CloneTLSCertificateTestCase(
    GmpCloneTLSCertificateTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224CreateTLSCertificateTestCase(
    GmpCreateTLSCertificateTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224DeleteTLSCertificateTestCase(
    GmpDeleteTLSCertificateTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetTLSCertificateTestCase(
    GmpGetTLSCertificateTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetTLSCertificatesTestCase(
    GmpGetTLSCertificatesTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyTLSCertificateTestCase(
    GmpModifyTLSCertificateTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.overrides import (
    GmpCloneOverrideTestMixin,
    GmpCreateOverrideTestMixin,
    GmpDeleteOverrideTestMixin,
    GmpGetOverridesTestMixin,
    GmpGetOverrideTestMixin,
    GmpModifyOverrideTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPCloneOverrideTestCase(GmpCloneOverrideTestMixin, GMPTestCase):
    pass


class GMPCreateOverrideTestCase(GmpCreateOverrideTestMixin, GMPTestCase):
    pass


class GMPDeleteOverrideTestCase(GmpDeleteOverrideTestMixin, GMPTestCase):
    pass


class GMPGetOverrideTestCase(GmpGetOverrideTestMixin, GMPTestCase):
    pass


class GMPGetOverridesTestCase(GmpGetOverridesTestMixin, GMPTestCase):
    pass


class GMPModifyOverrideTestCase(GmpModifyOverrideTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetTLSCertificatesTestMixin:
    def test_get_tls_certificates(self):
        self.gmp.get_tls_certificates()

        self.connection.send.has_been_called_with(b"<get_tls_certificates/>")

    def test_get_tls_certificates_with_filter_string(self):
        self.gmp.get_tls_certificates(filter_string="name=foo")

        self.connection.send.has_been_called_with(
            b'<get_tls_certificates filter="name=foo"/>'
        )

    def test_get_tls_certificates_with_include_certificate_data(self):
        self.gmp.get_tls_certificates(include_certificate_data="1")

        self.connection.send.has_been_called_with(
            b'<get_tls_certificates include_certificate_data="1"/>'
        )

    def test_get_tls_certificates_with_details(self):
        self.gmp.get_tls_certificates(details="1")

        self.connection.send.has_been_called_with(
            b'<get_tls_certificates details="1"/>'
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.tls_certificates import (
    GmpCloneTLSCertificateTestMixin,
    GmpCreateTLSCertificateTestMixin,
    GmpDeleteTLSCertificateTestMixin,
    GmpGetTLSCertificatesTestMixin,
    GmpGetTLSCertificateTestMixin,
    GmpModifyTLSCertificateTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225CloneTLSCertificateTestCase(
    GmpCloneTLSCertificateTestMixin, GMPTestCase
):
    pass


class Gmpv225CreateTLSCertificateTestCase(
    GmpCreateTLSCertificateTestMixin, GMPTestCase
):
    pass


class Gmpv225DeleteTLSCertificateTestCase(
    GmpDeleteTLSCertificateTestMixin, GMPTestCase
):
    pass


class Gmpv225GetTLSCertificateTestCase(
    GmpGetTLSCertificateTestMixin, GMPTestCase
):
    pass


class Gmpv225GetTLSCertificatesTestCase(
    GmpGetTLSCertificatesTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyTLSCertificateTestCase(
    GmpModifyTLSCertificateTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from collections.abc import Iterable

from gvm._enum import Enum
from gvm.errors import InvalidArgumentType, RequiredArgument
from gvm.protocols.core import Request
from gvm.xml import XmlCommand

from .._entity_id import EntityID
from ._entity_type import EntityType


class AggregateStatistic(Enum):
    """Enum for aggregate statistic types"""

    COUNT = "count"  # Number of items
    C_COUNT = "c_count"  # Cumulative number of items
    C_SUM = "c_sum"  # Cumulative sum of values
    MAX = "max"  # Maximum value
    MEAN = "mean"  # Arithmetic mean of values
    MIN = "min"  # Minimum value
    SUM = "sum"  # Sum of values
    TEXT = "text"  # Text column value
    VALUE = "value"  # Group or subgroup column value


class SortOrder(Enum):
    """Enum for sort order"""

    ASCENDING = "ascending"
    DESCENDING = "descending"


class Aggregates:
    @classmethod
    def get_aggregates(
        cls,
        resource_type: EntityType | str,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        sort_criteria: Iterable[dict[str, str | SortOrder | AggregateStatistic]]
        | None = None,
        data_columns: Iterable[str] | str | None = None,
        group_column: str | None = None,
        subgroup_column: str | None = None,
        text_columns: Iterable[str] | str | None = None,
        first_group: int | None = None,
        max_groups: int | None = None,
        mode: int | None = None,
        **kwargs,
    ) -> Request:
        """Request aggregated information on a resource / entity type

        Additional arguments can be set via the kwargs parameter for backward
        compatibility with older versions of python-gvm, but are not validated.

        Args:
            resource_type: The entity type to gather data from
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            sort_criteria: List of sort criteria (dicts that can contain
                a field, stat and order)
            data_columns: List of fields to aggregate data from
            group_column: The field to group the entities by
            subgroup_column: The field to further group the entities
                inside groups by
            text_columns: List of simple text columns which no statistics
                are calculated for
            first_group: The index of the first aggregate group to return
            max_groups: The maximum number of aggregate groups to return,
                -1 for all
            mode: Special mode for aggregation
        """
        if not resource_type:
            raise RequiredArgument(
                function=cls.get_aggregates.__name__, argument="resource_type"
            )

        if not isinstance(resource_type, EntityType):
            resource_type = EntityType(resource_type)

        cmd = XmlCommand("get_aggregates")

        _actual_resource_type = resource_type
        if resource_type.value == EntityType.AUDIT.value:
            _actual_resource_type = EntityType.TASK
            cmd.set_attribute("usage_type", "audit")
        elif resource_type.value == EntityType.POLICY.value:
            _actual_resource_type = EntityType.SCAN_CONFIG
            cmd.set_attribute("usage_type", "policy")
        elif resource_type.value == EntityType.SCAN_CONFIG.value:
            cmd.set_attribute("usage_type", "scan")
        elif resource_type.value == EntityType.TASK.value:
            cmd.set_attribute("usage_type", "scan")

        cmd.set_attribute("type", _actual_resource_type.value)

        cmd.add_filter(filter_string, filter_id)

        if first_group is not None:
            try:
                cmd.set_attribute("first_group", str(int(first_group)))
            except ValueError:
                raise InvalidArgumentType(
                    argument="first_group",
                    function=cls.get_aggregates.__name__,
                ) from None

        if max_groups is not None:
            try:
                cmd.set_attribute("max_groups", str(int(max_groups)))
            except ValueError:
                raise InvalidArgumentType(
                    argument="first_group",
                    function=cls.get_aggregates.__name__,
                )

        if sort_criteria is not None:
            if not isinstance(sort_criteria, Iterable) or isinstance(
                sort_criteria, str
            ):
                raise InvalidArgumentType(
                    argument="sort_criteria",
                    function=cls.get_aggregates.__name__,
                    arg_type=getattr(Iterable, "__name__", "Iterable"),
                )
            for sort in sort_criteria:
                if not isinstance(sort, dict):
                    raise InvalidArgumentType(
                        argument="sort_criteria",
                        function=cls.get_aggregates.__name__,
                    )

                sort_elem = cmd.add_element("sort")
                if sort.get("field"):
                    sort_elem.set_attribute("field", sort.get("field"))  # type: ignore

                if sort.get("stat"):
                    if isinstance(sort["stat"], AggregateStatistic):
                        sort_elem.set_attribute("stat", sort["stat"].value)
                    else:
                        stat = AggregateStatistic(sort["stat"])
                        sort_elem.set_attribute("stat", stat.value)

                if sort.get("order"):
                    if isinstance(sort["order"], SortOrder):
                        sort_elem.set_attribute("order", sort["order"].value)
                    else:
                        so = SortOrder(sort["order"])
                        sort_elem.set_attribute("order", so.value)

        if data_columns is not None:
            if isinstance(data_columns, str):
                data_columns = [data_columns]
            elif not isinstance(data_columns, Iterable):
                raise InvalidArgumentType(
                    function=cls.get_aggregates.__name__,
                    argument="data_columns",
                    arg_type=getattr(Iterable, "__name__", "Iterable"),
                )
            for column in data_columns:
                cmd.add_element("data_column", column)

        if group_column is not None:
            cmd.set_attribute("group_column", group_column)

        if subgroup_column is not None:
            if not group_column:
                raise RequiredArgument(
                    f"{cls.get_aggregates.__name__} requires a group_column"
                    " argument if subgroup_column is given",
                    function=cls.get_aggregates.__name__,
                    argument="subgroup_column",
                )
            cmd.set_attribute("subgroup_column", subgroup_column)

        if text_columns is not None:
            if isinstance(text_columns, str):
                text_columns = [text_columns]
            elif not isinstance(text_columns, Iterable):
                raise InvalidArgumentType(
                    function=cls.get_aggregates.__name__,
                    argument="text_columns",
                    arg_type=getattr(Iterable, "__name__", "Iterable"),
                )
            for column in text_columns:
                cmd.add_element("text_column", column)

        if mode is not None:
            cmd.set_attribute("mode", str(mode))

        # Add additional keyword args as attributes for backward compatibility.
        cmd.set_attributes(kwargs)

        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import GvmError


class GmpGetPolicyTestMixin:
    def test_get_policy(self):
        self.gmp.get_policy("a1")

        self.connection.send.has_been_called_with(
            b'<get_configs config_id="a1" usage_type="policy" details="1"/>'
        )

    def test_get_policy_with_audits(self):
        self.gmp.get_policy("a1", audits=True)

        self.connection.send.has_been_called_with(
            b'<get_configs config_id="a1" '
            b'usage_type="policy" tasks="1" details="1"/>'
        )

    def test_fail_without_policy_id(self):
        with self.assertRaises(GvmError):
            self.gmp.get_policy(None)

        with self.assertRaises(GvmError):
            self.gmp.get_policy("")



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.alerts import (
    GmpCloneAlertTestMixin,
    GmpCreateAlertTestMixin,
    GmpDeleteAlertTestMixin,
    GmpGetAlertsTestMixin,
    GmpGetAlertTestMixin,
    GmpModifyAlertTestMixin,
    GmpTestAlertTestMixin,
    GmpTriggerAlertTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225CloneAlertTestCase(GmpCloneAlertTestMixin, GMPTestCase):
    pass


class Gmpv225CreateAlertTestCase(GmpCreateAlertTestMixin, GMPTestCase):
    pass


class Gmpv225DeleteAlertTestCase(GmpDeleteAlertTestMixin, GMPTestCase):
    pass


class Gmpv225GetAlertTestCase(GmpGetAlertTestMixin, GMPTestCase):
    pass


class Gmpv225GetAlertsTestCase(GmpGetAlertsTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyAlertTestCase(GmpModifyAlertTestMixin, GMPTestCase):
    pass


class Gmpv225TestAlertTestCase(GmpTestAlertTestMixin, GMPTestCase):
    pass


class Gmpv225TriggerAlertTestCase(GmpTriggerAlertTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetPortListTestMixin:
    def test_get_port_list(self):
        self.gmp.get_port_list(port_list_id="port_list_id")

        self.connection.send.has_been_called_with(
            b'<get_port_lists port_list_id="port_list_id" details="1"/>'
        )

    def test_get_port_list_missing_port_list_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_port_list(port_list_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_port_list(port_list_id="")

        with self.assertRaises(RequiredArgument):
            self.gmp.get_port_list("")



#  SPDX-FileCopyrightText: 2026 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetIntegrationConfigsTestMixin:
    def test_get_integration_configs(self):
        self.gmp.get_integration_configs()

        self.connection.send.has_been_called_with(b"<get_integration_configs/>")

    def test_get_integration_configs_with_filter_string(self):
        self.gmp.get_integration_configs(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_integration_configs filter="foo=bar"/>'
        )

    def test_get_integration_configs_with_filter_id(self):
        self.gmp.get_integration_configs(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_integration_configs filt_id="f1"/>'
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.utils import is_list_like


class TestIsListLike(unittest.TestCase):
    def test_is_list_like(self):
        self.assertFalse(is_list_like(True))

        self.assertFalse(is_list_like("foo"))

        self.assertFalse(is_list_like(13))

        self.assertTrue(is_list_like([1]))

        self.assertTrue(is_list_like(["1", "2"]))

        self.assertTrue(is_list_like(("2", "3")))


if __name__ == "__main__":
    unittest.main()



from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.protocols.gmp.requests import EntityID
from gvm.utils import to_bool
from gvm.xml import XmlCommand


class IntegrationConfigs:
    @classmethod
    def get_integration_config(
        cls, integration_config_id: EntityID, *, details: bool | None = None
    ) -> Request:
        """Request a single Integration Configuration.

        Args:
            integration_config_id: UUID of the integration config to request.
            details: Whether to include detail information.
        """
        if not integration_config_id:
            raise RequiredArgument(
                function=cls.get_integration_config.__name__,
                argument="integration_config_id",
            )

        cmd = XmlCommand("get_integration_configs")
        cmd.set_attribute("integration_config_id", str(integration_config_id))

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def get_integration_configs(
        cls,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
    ) -> Request:
        """Request a list of Integration Configurations.

        Args:
            filter_string: Filter term to use for the query.
            filter_id: UUID of an existing filter to use for the query.
        """
        cmd = XmlCommand("get_integration_configs")
        cmd.add_filter(filter_string, filter_id)

        return cmd

    @classmethod
    def modify_integration_config(
        cls,
        integration_config_id: EntityID,
        *,
        service_url: str | None = None,
        service_cacert: str | None = None,
        oidc_provider_url: str | None = None,
        oidc_provider_client_id: str | None = None,
        oidc_provider_client_secret: str | None = None,
    ) -> Request:
        """Modify an existing Integration Configuration.

        Args:
            integration_config_id: UUID of configuration to modify.
            service_url: Integration Service URL.
            service_cacert: Integration Service Certificate.
            oidc_provider_url: OIDC Provider URL.
            oidc_provider_client_id: OIDC Provider Client ID.
            oidc_provider_client_secret: OIDC Provider Client Secret.
        """
        if not integration_config_id:
            raise RequiredArgument(
                function=cls.modify_integration_config.__name__,
                argument="integration_config_id",
            )

        cmd = XmlCommand("modify_integration_config")
        cmd.set_attribute("uuid", str(integration_config_id))

        # <service> element
        service = cmd.add_element("service")

        if service_url is not None:
            service.add_element("url", service_url)
        else:
            service.add_element("url")
        if service_cacert is not None:
            service.add_element("cacert", service_cacert)
        else:
            service.add_element("cacert")

        # <oidc> element
        oidc = cmd.add_element("oidc")
        if oidc_provider_url is not None:
            oidc.add_element("oidc_provider_url", oidc_provider_url)
        else:
            oidc.add_element("oidc_provider_url")

        oidc_client = oidc.add_element("client")
        if oidc_provider_client_id is not None:
            oidc_client.add_element("id", oidc_provider_client_id)
        else:
            oidc_client.add_element("id")
        if oidc_provider_client_secret is not None:
            oidc_client.add_element("secret", oidc_provider_client_secret)
        else:
            oidc_client.add_element("secret")

        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetOperatingSystemTestMixin:
    def test_get_operating_system(self):
        self.gmp.get_operating_system("a1")

        self.connection.send.has_been_called_with(
            b'<get_assets asset_id="a1" type="os"/>'
        )
        self.gmp.get_operating_system(operating_system_id="a1")

        self.connection.send.has_been_called_with(
            b'<get_assets asset_id="a1" type="os"/>'
        )

    def test_get_operating_system_details(self):
        self.gmp.get_operating_system("a1", details=True)

        self.connection.send.has_been_called_with(
            b'<get_assets asset_id="a1" type="os" details="1"/>'
        )
        self.gmp.get_operating_system("a1", details=False)

        self.connection.send.has_been_called_with(
            b'<get_assets asset_id="a1" type="os" details="0"/>'
        )

    def test_get_asset_missing_operating_system_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_operating_system(operating_system_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_operating_system(operating_system_id="")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import AliveTest


class GmpCreateTargetTestMixin:
    def test_create_target_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_target(None, hosts=["foo"])

        with self.assertRaises(RequiredArgument):
            self.gmp.create_target(name=None, hosts=["foo"])

        with self.assertRaises(RequiredArgument):
            self.gmp.create_target("", hosts=["foo"])

    def test_create_target_with_asset_hosts_filter(self):
        self.gmp.create_target("foo", asset_hosts_filter="name=foo")

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b'<asset_hosts filter="name=foo"/>'
            b"</create_target>"
        )

    def test_create_target_missing_hosts(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_target(name="foo")

    def test_create_target_with_comment(self):
        self.gmp.create_target("foo", hosts=["foo"], comment="bar")

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b"<comment>bar</comment>"
            b"</create_target>"
        )

    def test_create_target_with_exclude_hosts(self):
        self.gmp.create_target(
            "foo", hosts=["foo", "bar"], exclude_hosts=["bar", "ipsum"]
        )

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo,bar</hosts>"
            b"<exclude_hosts>bar,ipsum</exclude_hosts>"
            b"</create_target>"
        )

    def test_create_target_with_ssh_credential(self):
        self.gmp.create_target("foo", hosts=["foo"], ssh_credential_id="c1")

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b'<ssh_credential id="c1"/>'
            b"</create_target>"
        )

    def test_create_target_with_ssh_credential_port(self):
        self.gmp.create_target(
            "foo",
            hosts=["foo"],
            ssh_credential_id="c1",
            ssh_credential_port=123,
        )

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b'<ssh_credential id="c1">'
            b"<port>123</port>"
            b"</ssh_credential>"
            b"</create_target>"
        )

    def test_create_target_with_smb_credential_id(self):
        self.gmp.create_target("foo", hosts=["foo"], smb_credential_id="c1")

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b'<smb_credential id="c1"/>'
            b"</create_target>"
        )

    def test_create_target_with_esxi_credential_id(self):
        self.gmp.create_target("foo", hosts=["foo"], esxi_credential_id="c1")

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b'<esxi_credential id="c1"/>'
            b"</create_target>"
        )

    def test_create_target_with_snmp_credential_id(self):
        self.gmp.create_target("foo", hosts=["foo"], snmp_credential_id="c1")

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b'<snmp_credential id="c1"/>'
            b"</create_target>"
        )

    def test_create_target_with_alive_tests(self):
        self.gmp.create_target(
            "foo", hosts=["foo"], alive_test=AliveTest.ICMP_PING
        )

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b"<alive_tests>ICMP Ping</alive_tests>"
            b"</create_target>"
        )

    def test_create_target_invalid_alive_tests(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.create_target("foo", hosts=["foo"], alive_test="foo")

    def test_create_target_with_allow_simultaneous_ips(self):
        self.gmp.create_target(
            "foo", hosts=["foo"], allow_simultaneous_ips=True
        )

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b"<allow_simultaneous_ips>1</allow_simultaneous_ips>"
            b"</create_target>"
        )

        self.gmp.create_target(
            "foo", hosts=["foo"], allow_simultaneous_ips=False
        )

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b"<allow_simultaneous_ips>0</allow_simultaneous_ips>"
            b"</create_target>"
        )

    def test_create_target_with_reverse_lookup_only(self):
        self.gmp.create_target("foo", hosts=["foo"], reverse_lookup_only=True)

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b"<reverse_lookup_only>1</reverse_lookup_only>"
            b"</create_target>"
        )

        self.gmp.create_target("foo", hosts=["foo"], reverse_lookup_only=False)

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b"<reverse_lookup_only>0</reverse_lookup_only>"
            b"</create_target>"
        )

    def test_create_target_with_reverse_lookup_unify(self):
        self.gmp.create_target("foo", hosts=["foo"], reverse_lookup_unify=True)

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b"<reverse_lookup_unify>1</reverse_lookup_unify>"
            b"</create_target>"
        )

        self.gmp.create_target("foo", hosts=["foo"], reverse_lookup_unify=False)

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b"<reverse_lookup_unify>0</reverse_lookup_unify>"
            b"</create_target>"
        )

    def test_create_target_with_port_range(self):
        self.gmp.create_target("foo", hosts=["foo"], port_range="bar")

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b"<port_range>bar</port_range>"
            b"</create_target>"
        )

    def test_create_target_with_port_list_id(self):
        self.gmp.create_target("foo", hosts=["foo"], port_list_id="pl1")

        self.connection.send.has_been_called_with(
            b"<create_target>"
            b"<name>foo</name>"
            b"<hosts>foo</hosts>"
            b'<port_list id="pl1"/>'
            b"</create_target>"
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import sys
from io import IOBase
from typing import AnyStr, TextIO
from uuid import UUID

from lxml.etree import DocInfo, SubElement, XMLParser
from lxml.etree import Element as create_element  # noqa: N813
from lxml.etree import Error as EtreeError
from lxml.etree import _Element as Element
from lxml.etree import iselement as is_xml_element
from lxml.etree import tostring as xml_to_string

from gvm.errors import GvmError, InvalidArgumentType

__all__ = (
    "Element",
    "XmlCommand",
    "XmlCommandElement",
    "create_parser",
    "parse_xml",
    "pretty_print",
)


class XmlError(GvmError):
    pass


def check_xml_document(
    root_element: Element,
    forbid_dtd: bool = False,
    forbid_entities: bool = True,
) -> None:
    """
    Check an element for DTD and entity declarations
    """
    doc_info: DocInfo = root_element.getroottree().docinfo
    if doc_info.doctype:  # type: ignore
        if forbid_dtd:
            raise XmlError(
                "XML document contains a forbidden DTD declaration "
                f"{doc_info.system_url} {doc_info.public_id}"  # type: ignore
            )

    if forbid_entities:
        for dtd in doc_info.internalDTD, doc_info.externalDTD:
            if dtd is None:
                continue
            for entity in dtd.iterentities():  # type: ignore
                raise XmlError(
                    f"XML Document contains forbidden entity declaration "
                    f"{entity.name} {entity.content}"
                )


def create_parser():
    # huge_tree => disable security restrictions and support very deep trees and
    #              very long text content (for get_reports)
    # resolve_entities=False => disable entity resolution for security reasons
    return XMLParser(encoding="utf-8", huge_tree=True, resolve_entities=False)


def parse_xml(xml: AnyStr) -> Element:
    """
    Parse an XML string and return the root element

    Raises an XmlError if the XML is invalid.
    """
    parser = create_parser()
    try:
        parser.feed(xml)
        element = parser.close()
        check_xml_document(element)
        return element
    except EtreeError as e:
        raise XmlError(f"Invalid XML {xml!r}. Error was {e}") from e


class XmlCommandElement:
    """
    Base class for XML commands

    It's used to create XML command requests for the XML based protocols.
    """

    def __init__(self, element: Element):
        self._element = element

    def add_element(
        self,
        name: str,
        text: str | None = None,
        *,
        attrs: dict | None = None,
    ) -> "XmlCommandElement":
        node = SubElement(self._element, name, attrib=attrs)
        node.text = text
        return XmlCommandElement(node)

    def set_attribute(self, name: str, value: str) -> "XmlCommandElement":
        """Set an attribute on the element.

        Args:
            name: Name of the attribute
            value: Value of the attribute
        """
        self._element.set(name, value)
        return self

    def set_attributes(self, attrs: dict[str, str]) -> "XmlCommandElement":
        """Set several attributes at once.

        Args:
            attrs: Attributes to be set on the element
        """
        for key, value in attrs.items():
            self._element.set(key, value)

        return self

    def set_text(self, text: str | None) -> "XmlCommandElement":
        """Set the text of the element.

        Args:
            text: Text to be set on the element. None to remove the text.
        """
        self._element.text = text
        return self

    def append_xml_str(self, xml_text: str) -> None:
        """Append a xml element in string format."""
        node = parse_xml(xml_text)
        self._element.append(node)

    def to_string(self, *, encoding: str = "utf-8") -> str:
        """
        Convert the XML element to a string

        Args:
            encoding: The encoding to use for the string. Default is 'utf-8'.
        """
        return self.to_bytes().decode(encoding)

    def to_bytes(self) -> bytes:
        """
        Convert the XML element to a bytes object
        """
        return xml_to_string(self._element)

    def __str__(self) -> str:
        """
        Convert the XML element to a string using the default encoding.
        """
        return self.to_string()

    def __bytes__(self) -> bytes:
        """
        Convert the XML element to a bytes object
        """
        return self.to_bytes()


class XmlCommand(XmlCommandElement):
    """
    Class to create XML commands
    """

    def __init__(self, name: str) -> None:
        """
        Create a new XML command

        Args:
            name: The name of the root element of the command.
        """
        super().__init__(create_element(name))

    def add_filter(
        self,
        filter_string: str | None,
        filter_id: str | UUID | None,
    ) -> "XmlCommand":
        """
        Add a filter to the command.

        Args:
            filter_string: The filter string to be added.
            filter_id: The filter ID to be added.
        """
        if filter_string:
            self.set_attribute("filter", filter_string)

        if filter_id:
            self.set_attribute("filt_id", str(filter_id))

        return self


def pretty_print(
    xml: str | list[Element | str] | Element,
    file: TextIO | IOBase = sys.stdout,
):
    """Prints beautiful XML-Code

    This function gets a string containing the xml, an object of
    List[lxml.etree.Element] or directly a lxml element.

    Print it with good readable format.

    Arguments:
        xml (str, List[lxml.etree.Element] or lxml.etree.Element):
            xml as string,
            List[lxml.etree.Element] or directly a lxml element.
        file:
            A IOBase type. Can be a File, StringIO, ...

    """
    if not isinstance(file, (IOBase, TextIO)):
        raise TypeError(
            f"Type needs to be from IOBase or TextIO, not {type(file)}."
        ) from None

    if isinstance(xml, list):
        for item in xml:
            if is_xml_element(item):
                file.write(
                    xml_to_string(item, pretty_print=True).decode(
                        sys.getdefaultencoding() + "\n"
                    )
                )
            else:
                file.write(str(item) + "\n")
    elif is_xml_element(xml):
        file.write(
            xml_to_string(xml, pretty_print=True).decode(
                sys.getdefaultencoding() + "\n"
            )
        )
    elif isinstance(xml, str):
        tree = parse_xml(xml)
        file.write(
            xml_to_string(tree, pretty_print=True).decode(
                sys.getdefaultencoding() + "\n"
            )
        )
    else:
        raise InvalidArgumentType(
            function=pretty_print.__name__, argument="xml", arg_type=""
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v225 import HostsOrdering


class GetHostsOrderingFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            HostsOrdering.from_string("foo")

    def test_none_or_empty(self):
        ct = HostsOrdering.from_string(None)
        self.assertIsNone(ct)
        ct = HostsOrdering.from_string("")
        self.assertIsNone(ct)

    def test_sequential(self):
        ct = HostsOrdering.from_string("sequential")
        self.assertEqual(ct, HostsOrdering.SEQUENTIAL)

    def test_random(self):
        ct = HostsOrdering.from_string("random")
        self.assertEqual(ct, HostsOrdering.RANDOM)

    def test_reverse(self):
        ct = HostsOrdering.from_string("reverse")
        self.assertEqual(ct, HostsOrdering.REVERSE)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import AnyStr, Protocol

from lxml import etree

from gvm.errors import GvmError

from ._request import Request
from ._response import Response


class XmlReader:
    """
    Read a XML command until its closing element
    """

    def start_xml(self) -> None:
        self._first_element: etree._Element | None = None
        # act on start and end element events and
        # allow huge text data (for report content)
        self._parser = etree.XMLPullParser(
            events=("start", "end"), huge_tree=True
        )

    def is_end_xml(self) -> bool:
        for action, obj in self._parser.read_events():
            if not self._first_element and action in "start":
                self._first_element = obj.tag  # type: ignore

            if (
                self._first_element
                and action in "end"
                and str(self._first_element) == str(obj.tag)  # type: ignore
            ):
                return True
        return False

    def feed_xml(self, data: AnyStr) -> None:
        try:
            self._parser.feed(data)
        except etree.ParseError as e:
            raise GvmError(
                f"Cannot parse XML response. Response data read {data!r}",
                e,
            ) from None


class InvalidStateError(GvmError):
    """
    Error raised if the Connection would be moved into an invalid state
    """

    def __init__(self, message: str = "Invalid State", *args):
        super().__init__(message, *args)


class State(Protocol):
    def __set_context__(self, context: "Context") -> None: ...
    def send(self, request: Request) -> bytes: ...
    def receive_data(self, data: bytes) -> Response | None: ...
    def close(self) -> None: ...


class Context(Protocol):
    def __set_state__(self, state: State) -> None: ...


class AbstractState:
    _context: Context

    def __set_context__(self, context: Context) -> None:
        self._context = context

    def set_next_state(self, next_state: State) -> None:
        self._context.__set_state__(next_state)


class InitialState(AbstractState):
    def send(self, request: Request) -> bytes:
        self.set_next_state(AwaitingResponseState(request))
        return bytes(request)

    def receive_data(self, data: bytes) -> Response | None:
        raise InvalidStateError()

    def close(self) -> None:
        # nothing to do
        return


class AwaitingResponseState(AbstractState):
    def __init__(self, request: Request) -> None:
        self._request = request

    def send(self, request: Request) -> bytes:
        raise InvalidStateError()

    def close(self) -> None:
        self.set_next_state(InitialState())

    def receive_data(self, data: bytes) -> Response | None:
        next_state = ReceivingDataState(self._request)
        self.set_next_state(next_state)
        return next_state.receive_data(data)


class ErrorState(AbstractState):
    message = (
        "The connection is in an error state. Please close the connection."
    )

    def send(self, request: Request) -> bytes:
        raise InvalidStateError(self.message)

    def close(self) -> None:
        self.set_next_state(InitialState())

    def receive_data(self, data: bytes) -> Response | None:
        raise InvalidStateError(self.message)


class ReceivingDataState(AbstractState):
    def __init__(self, request: Request) -> None:
        self._request = request
        self._data = bytearray()
        self._reader = XmlReader()
        self._reader.start_xml()

    def send(self, request: Request) -> bytes:
        raise InvalidStateError()

    def close(self) -> None:
        self.set_next_state(InitialState())

    def receive_data(self, data: bytes) -> Response | None:
        self._data += data
        try:
            self._reader.feed_xml(data)
        except GvmError as e:
            self.set_next_state(ErrorState())
            raise e

        if not self._reader.is_end_xml():
            return None

        self.set_next_state(InitialState())
        return Response(data=bytes(self._data), request=self._request)


class Connection:
    """
    This is a [SansIO](https://sans-io.readthedocs.io) connection for GMP

    It is responsible for creating bytes from GMP XML requests and transforming
    XML response data into GMP responses.
    """

    def __init__(self) -> None:
        self.__set_state__(InitialState())

    def send(self, request: Request) -> bytes:
        """
        Create data from a request to be send

        Returns:
            The data for a request that can be send for example over a socket

        Raises:
            An InvalidStateError if no request can be send currently. For
            example when waiting for a response to a previous request.
        """
        return self._state.send(request)

    def receive_data(self, data: bytes) -> Response | None:
        """
        Feed received data a response is complete

        Returns:
            A Response if the response data is complete and None if data is
            still to be received.

        Raises:
            An InvalidStateError if no data can be received currently. For
            example if no request is send yet.
        """
        return self._state.receive_data(data)

    def close(self) -> None:
        """
        Close the connection and reset the state of the protocol

        Afterwards the connection can be reused for sending a new request.
        """
        return self._state.close()

    def __set_state__(self, state: State) -> None:
        self._state = state
        self._state.__set_context__(self)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gvm.protocols.core import Request
from gvm.xml import XmlCommand


class Version:
    @staticmethod
    def get_version() -> Request:
        """Get the Greenbone Vulnerability Management Protocol (GMP) version
        used by the remote gvmd.
        """
        return XmlCommand("get_version")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class TrashCan:
    @staticmethod
    def empty_trashcan() -> Request:
        """Empty the trashcan

        Remove all entities from the trashcan. **Attention:** this command can
        not be reverted
        """
        return XmlCommand("empty_trashcan")

    @classmethod
    def restore_from_trashcan(cls, entity_id: EntityID) -> Request:
        """Restore an entity from the trashcan

        Args:
            entity_id: ID of the entity to be restored from the trashcan
        """

        if not entity_id:
            raise RequiredArgument(
                function=cls.restore_from_trashcan.__name__,
                argument="entity_id",
            )

        cmd = XmlCommand("restore")
        cmd.set_attribute("id", str(entity_id))

        return cmd



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.system.user_settings import (
    GmpGetUserSettingsTestMixin,
    GmpGetUserSettingTestMixin,
    GmpModifyUserSettingTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225GetUserSettingTestCase(GmpGetUserSettingTestMixin, GMPTestCase):
    pass


class Gmpv225GetUserSettingsTestCase(GmpGetUserSettingsTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyUserSettingTestCase(
    GmpModifyUserSettingTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


from gvm.protocols.gmp import GMPv226

from .. import GmpTestCase as BaseGMPTestCase


class GMPTestCase(BaseGMPTestCase):
    gmp_class = GMPv226



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.protocols.gmp.requests.v225 import HostsOrdering

from . import GMPTestCase


class GmpWithStatementTestMixin:
    def test_types(self):
        with self.gmp:
            # Test that the values are equal
            self.assertEqual(
                self.gmp.types.AlertEvent.TASK_RUN_STATUS_CHANGED.value,
                "Task run status changed",
            )
            self.assertEqual(
                self.gmp.types.PermissionSubjectType.USER.value, "user"
            )
            self.assertEqual(
                self.gmp.types.HostsOrdering.RANDOM.value, "random"
            )

            # Test usability of from_string
            self.assertEqual(
                self.gmp.types.HostsOrdering.from_string("reverse"),
                self.gmp.types.HostsOrdering.REVERSE,
            )

            # Test, that the Enum class types are equal
            self.assertEqual(self.gmp.types.HostsOrdering, HostsOrdering)


class Gmpv225WithStatementTestCase(GmpWithStatementTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCreateTLSCertificateTestMixin:
    def test_create_tls_certificate(self):
        self.gmp.create_tls_certificate("foo", "c1", comment="bar", trust=True)

        self.connection.send.has_been_called_with(
            b"<create_tls_certificate>"
            b"<name>foo</name>"
            b"<certificate>c1</certificate>"
            b"<comment>bar</comment>"
            b"<trust>1</trust>"
            b"</create_tls_certificate>"
        )

        self.gmp.create_tls_certificate("foo", "c1", trust=False)

        self.connection.send.has_been_called_with(
            b"<create_tls_certificate>"
            b"<name>foo</name>"
            b"<certificate>c1</certificate>"
            b"<trust>0</trust>"
            b"</create_tls_certificate>"
        )

    def test_missing_certificate(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_tls_certificate(name="foo", certificate="")

        with self.assertRaises(RequiredArgument):
            self.gmp.create_tls_certificate(name="foo", certificate=None)

    def test_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_tls_certificate(name=None, certificate="c1")

        with self.assertRaises(RequiredArgument):
            self.gmp.create_tls_certificate(name="", certificate="c1")



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetTicketTestMixin:
    def test_get_ticket(self):
        self.gmp.get_ticket("t1")

        self.connection.send.has_been_called_with(
            b'<get_tickets ticket_id="t1"/>'
        )

        self.gmp.get_ticket(ticket_id="t1")

        self.connection.send.has_been_called_with(
            b'<get_tickets ticket_id="t1"/>'
        )

    def test_get_ticket_missing_user_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_ticket(ticket_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_ticket("")



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest
from unittest.mock import MagicMock, patch

from gvm.protocols.http.openvasd import OpenvasdHttpAPIv1


class TestOpenvasdHttpApiV1(unittest.TestCase):
    @patch("gvm.protocols.http.openvasd._openvasd1.create_openvasd_http_client")
    def test_initializes_all_sub_apis(self, mock_crate_openvasd_client):
        mock_httpx_client = MagicMock()
        mock_crate_openvasd_client.return_value = mock_httpx_client

        api = OpenvasdHttpAPIv1(
            host_name="localhost",
            port=3000,
            api_key="test-key",
            server_ca_path="/path/to/ca.pem",
            client_cert_paths=("/path/to/client.pem", "/path/to/key.pem"),
        )

        mock_crate_openvasd_client.assert_called_once_with(
            host_name="localhost",
            port=3000,
            api_key="test-key",
            server_ca_path="/path/to/ca.pem",
            client_cert_paths=("/path/to/client.pem", "/path/to/key.pem"),
        )

        self.assertEqual(api._client, mock_httpx_client)
        self.assertEqual(api.health._client, mock_httpx_client)
        self.assertEqual(api.metadata._client, mock_httpx_client)
        self.assertEqual(api.notus._client, mock_httpx_client)
        self.assertEqual(api.scans._client, mock_httpx_client)
        self.assertEqual(api.vts._client, mock_httpx_client)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
GMP Request implementations for GMP version 22.4.
"""

from .._entity_id import EntityID
from .._version import Version
from ._aggregates import Aggregates, AggregateStatistic, SortOrder
from ._alerts import AlertCondition, AlertEvent, AlertMethod, Alerts
from ._audits import Audits
from ._auth import Authentication
from ._cert_bund_advisories import CertBundAdvisories
from ._cpes import Cpes
from ._credentials import (
    CredentialFormat,
    Credentials,
    CredentialType,
    SnmpAuthAlgorithm,
    SnmpPrivacyAlgorithm,
)
from ._cves import Cves
from ._dfn_cert_advisories import DfnCertAdvisories
from ._entity_type import EntityType
from ._feed import Feed, FeedType
from ._filters import Filters, FilterType
from ._groups import Groups
from ._help import Help, HelpFormat
from ._hosts import Hosts, HostsOrdering
from ._notes import Notes
from ._nvts import Nvts
from ._operating_systems import OperatingSystems
from ._overrides import Overrides
from ._permissions import Permissions, PermissionSubjectType
from ._policies import Policies
from ._port_lists import PortLists, PortRangeType
from ._report_formats import ReportFormats, ReportFormatType
from ._reports import Reports
from ._results import Results
from ._roles import Roles
from ._scan_configs import ScanConfigs
from ._scanners import Scanners, ScannerType
from ._schedules import Schedules
from ._secinfo import InfoType, SecInfo
from ._severity import Severity
from ._system_reports import SystemReports
from ._tags import Tags
from ._targets import AliveTest, Targets
from ._tasks import Tasks
from ._tickets import Tickets, TicketStatus
from ._tls_certificates import TLSCertificates
from ._trashcan import TrashCan
from ._user_settings import UserSettings
from ._users import UserAuthType, Users
from ._vulnerabilities import Vulnerabilities

__all__ = (
    "AggregateStatistic",
    "Aggregates",
    "AlertCondition",
    "AlertEvent",
    "AlertMethod",
    "Alerts",
    "AliveTest",
    "Audits",
    "Authentication",
    "CertBundAdvisories",
    "Cpes",
    "CredentialFormat",
    "CredentialType",
    "Credentials",
    "Cves",
    "DfnCertAdvisories",
    "EntityID",
    "EntityType",
    "Feed",
    "FeedType",
    "FilterType",
    "Filters",
    "Groups",
    "Help",
    "HelpFormat",
    "Hosts",
    "HostsOrdering",
    "InfoType",
    "Notes",
    "Nvts",
    "OperatingSystems",
    "Overrides",
    "PermissionSubjectType",
    "Permissions",
    "Policies",
    "PortLists",
    "PortRangeType",
    "ReportFormatType",
    "ReportFormats",
    "Reports",
    "Results",
    "Roles",
    "ScanConfigs",
    "ScannerType",
    "Scanners",
    "Schedules",
    "SecInfo",
    "Severity",
    "SnmpAuthAlgorithm",
    "SnmpPrivacyAlgorithm",
    "SortOrder",
    "SystemReports",
    "TLSCertificates",
    "Tags",
    "Targets",
    "Tasks",
    "TicketStatus",
    "Tickets",
    "TrashCan",
    "UserAuthType",
    "UserSettings",
    "Users",
    "Version",
    "Vulnerabilities",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetScanConfigNvtsTestMixin:
    def test_get_scan_config_nvts_simple(self):
        self.gmp.get_scan_config_nvts()

        self.connection.send.has_been_called_with(b"<get_nvts/>")

    def test_get_scan_config_nvts_with_details(self):
        self.gmp.get_scan_config_nvts(details=True)

        self.connection.send.has_been_called_with(b'<get_nvts details="1"/>')

        self.gmp.get_scan_config_nvts(details=False)

        self.connection.send.has_been_called_with(b'<get_nvts details="0"/>')

    def test_get_scan_config_nvts_with_preferences(self):
        self.gmp.get_scan_config_nvts(preferences=True)

        self.connection.send.has_been_called_with(
            b'<get_nvts preferences="1"/>'
        )

        self.gmp.get_scan_config_nvts(preferences=False)

        self.connection.send.has_been_called_with(
            b'<get_nvts preferences="0"/>'
        )

    def test_get_scan_config_nvts_with_preference_count(self):
        self.gmp.get_scan_config_nvts(preference_count=True)

        self.connection.send.has_been_called_with(
            b'<get_nvts preference_count="1"/>'
        )

    def test_get_scan_config_nvts_with_timeout(self):
        self.gmp.get_scan_config_nvts(timeout=True)

        self.connection.send.has_been_called_with(b'<get_nvts timeout="1"/>')

        self.gmp.get_scan_config_nvts(timeout=False)

        self.connection.send.has_been_called_with(b'<get_nvts timeout="0"/>')

    def test_get_scan_config_nvts_with_config_id(self):
        self.gmp.get_scan_config_nvts(config_id="config_id")

        self.connection.send.has_been_called_with(
            b'<get_nvts config_id="config_id"/>'
        )

    def test_get_scan_config_nvts_with_preferences_config_id(self):
        self.gmp.get_scan_config_nvts(
            preferences_config_id="preferences_config_id"
        )

        self.connection.send.has_been_called_with(
            b'<get_nvts preferences_config_id="preferences_config_id"/>'
        )

    def test_get_scan_config_nvts_with_family(self):
        self.gmp.get_scan_config_nvts(family="family")

        self.connection.send.has_been_called_with(
            b'<get_nvts family="family"/>'
        )

    def test_get_scan_config_nvts_with_sort_order(self):
        self.gmp.get_scan_config_nvts(sort_order="sort_order")

        self.connection.send.has_been_called_with(
            b'<get_nvts sort_order="sort_order"/>'
        )

    def test_get_scan_config_nvts_with_sort_field(self):
        self.gmp.get_scan_config_nvts(sort_field="sort_field")

        self.connection.send.has_been_called_with(
            b'<get_nvts sort_field="sort_field"/>'
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .aggregates import GmpGetAggregatesTestMixin


class Gmpv224GetAggregatesTestCase(GmpGetAggregatesTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gvm.errors import RequiredArgument


class GmpModifyAgentGroupTestMixin:
    def test_modify_agent_group(self):
        self.gmp.modify_agent_group(
            agent_group_id="group-1",
            name="NewName",
            comment="Updated comment",
            agent_ids=["agent-1", "agent-2"],
        )

        self.connection.send.has_been_called_with(
            b'<modify_agent_group agent_group_id="group-1">'
            b"<name>NewName</name>"
            b"<comment>Updated comment</comment>"
            b'<agents><agent id="agent-1"/><agent id="agent-2"/></agents>'
            b"</modify_agent_group>"
        )

    def test_modify_agent_group_without_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agent_group(agent_group_id=None)

    def test_modify_agent_group_without_name(self):
        self.gmp.modify_agent_group(
            agent_group_id="group-1",
            comment="Updated comment",
            agent_ids=["agent-1", "agent-2"],
        )

        self.connection.send.has_been_called_with(
            b'<modify_agent_group agent_group_id="group-1">'
            b"<comment>Updated comment</comment>"
            b'<agents><agent id="agent-1"/><agent id="agent-2"/></agents>'
            b"</modify_agent_group>"
        )

    def test_modify_agent_group_without_comment(self):
        self.gmp.modify_agent_group(
            agent_group_id="group-1",
            name="NewName",
            agent_ids=["agent-1", "agent-2"],
        )

        self.connection.send.has_been_called_with(
            b'<modify_agent_group agent_group_id="group-1">'
            b"<name>NewName</name>"
            b'<agents><agent id="agent-1"/><agent id="agent-2"/></agents>'
            b"</modify_agent_group>"
        )

    def test_modify_agent_group_without_agent_ids(self):
        self.gmp.modify_agent_group(
            agent_group_id="group-1",
            name="NewName",
            comment="Updated comment",
        )

        self.connection.send.has_been_called_with(
            b'<modify_agent_group agent_group_id="group-1">'
            b"<name>NewName</name>"
            b"<comment>Updated comment</comment>"
            b"</modify_agent_group>"
        )



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.protocols.gmp.requests._entity_id import EntityID
from gvm.utils import to_bool
from gvm.xml import XmlCommand


class AgentGroups:
    @classmethod
    def create_agent_group(
        cls,
        name: str,
        agent_ids: list[str],
        *,
        comment: str | None = None,
    ) -> Request:
        """Create a new agent group.

        Args:
            name: Name of the new agent group.
            agent_ids: List of agent UUIDs to include in the group (required).
            comment: Optional comment for the group.

        Raises:
            RequiredArgument: If name or agent_ids is not provided.
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_agent_group.__name__, argument="name"
            )

        if not agent_ids:
            raise RequiredArgument(
                function=cls.create_agent_group.__name__, argument="agent_ids"
            )

        cmd = XmlCommand("create_agent_group")
        cmd.add_element("name", name)

        if comment:
            cmd.add_element("comment", comment)

        agents_element = cmd.add_element("agents")
        for agent_id in agent_ids:
            agents_element.add_element("agent", attrs={"id": agent_id})

        return cmd

    @classmethod
    def clone_agent_group(cls, agent_group_id: EntityID) -> Request:
        """Clone an existing agent group

        Args:
            agent_group_id: UUID of an existing agent group to clone from

        Returns:
            Request: GMP command to create a new agent group based on a copy
        """
        if not agent_group_id:
            raise RequiredArgument(
                function=cls.clone_agent_group.__name__,
                argument="agent_group_id",
            )

        cmd = XmlCommand("create_agent_group")
        cmd.add_element("copy", str(agent_group_id))

        return cmd

    @classmethod
    def modify_agent_group(
        cls,
        agent_group_id: EntityID,
        *,
        name: str | None = None,
        comment: str | None = None,
        agent_ids: list[str] | None = None,
    ) -> Request:
        """Modify an existing agent group.

        Args:
            agent_group_id: UUID of the group to modify.
            name: Optional new name for the group.
            comment: Optional comment for the group.
            agent_ids: Optional list of agent UUIDs to set for the group.

        Raises:
            RequiredArgument: If agent_group_id is not provided.
        """
        if not agent_group_id:
            raise RequiredArgument(
                function=cls.modify_agent_group.__name__,
                argument="agent_group_id",
            )

        cmd = XmlCommand("modify_agent_group")
        cmd.set_attribute("agent_group_id", str(agent_group_id))

        if name:
            cmd.add_element("name", name)

        if comment:
            cmd.add_element("comment", comment)

        if agent_ids:
            agents_element = cmd.add_element("agents")
            for agent_id in agent_ids:
                agents_element.add_element("agent", attrs={"id": agent_id})

        return cmd

    @classmethod
    def delete_agent_group(
        cls,
        agent_group_id: EntityID,
        *,
        ultimate: bool | None = False,
    ) -> Request:
        """Delete an existing agent group.

        Args:
            agent_group_id: UUID of the group to delete.
            ultimate: Whether to permanently delete or move to trashcan.

        Raises:
            RequiredArgument: If agent_group_id is not provided.
        """
        if not agent_group_id:
            raise RequiredArgument(
                function=cls.delete_agent_group.__name__,
                argument="agent_group_id",
            )

        cmd = XmlCommand("delete_agent_group")
        cmd.set_attribute("agent_group_id", str(agent_group_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @classmethod
    def get_agent_groups(
        cls,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
    ) -> Request:
        """Request a list of agent groups.

        Args:
            filter_string: Filter expression to use.
            filter_id: UUID of a predefined filter.
            trash: If True, return trashed agent groups.

        Returns:
            Request object to fetch agent groups.
        """
        cmd = XmlCommand("get_agent_groups")
        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        return cmd

    @classmethod
    def get_agent_group(cls, agent_group_id: EntityID) -> Request:
        """Request a single agent group by ID.

        Args:
            agent_group_id: UUID of the agent group.

        Raises:
            RequiredArgument: If agent_group_id is not provided.

        Returns:
            Request object to fetch the specific agent group.
        """
        if not agent_group_id:
            raise RequiredArgument(
                function=cls.get_agent_group.__name__, argument="agent_group_id"
            )

        cmd = XmlCommand("get_agent_groups")
        cmd.set_attribute("agent_group_id", str(agent_group_id))

        return cmd



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from base64 import b64encode

from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class CredentialStores:
    @classmethod
    def get_credential_store(
        cls,
        credential_store_id: EntityID,
        *,
        details: bool | None = None,
    ) -> Request:
        """Request a credential store

        Args:
            credential_store_id: ID of credential store to fetch
            details: True to request all details
        """

        if not credential_store_id:
            raise RequiredArgument(
                function=cls.get_credential_store.__name__,
                argument="credential_store_id",
            )

        cmd = XmlCommand("get_credential_stores")
        cmd.add_element("credential_store_id", str(credential_store_id))

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def get_credential_stores(
        cls,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        details: bool | None = None,
    ) -> Request:
        """Request a list of credential stores

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            details: True to request all details
        """

        cmd = XmlCommand("get_credential_stores")
        cmd.add_filter(filter_string, filter_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        return cmd

    @classmethod
    def modify_credential_store(
        cls,
        credential_store_id: EntityID,
        *,
        active: bool | None = None,
        host: str | None = None,
        port: int | None = None,
        path: str | None = None,
        app_id: str | None = None,
        client_cert: str | None = None,
        client_key: str | None = None,
        client_pkcs12_file: str | None = None,
        passphrase: str | None = None,
        server_ca_cert: str | None = None,
        comment: str | None = None,
    ) -> Request:
        """Modify a credential store

        Args:
            credential_store_id: ID of credential store to fetch
            active: Whether the credential store is active
            host: The host to use for reaching the credential store
            port: The port to use for reaching the credential store
            path: The URI path the credential store is using
            app_id: Depends on the credential store used. Usually called the same in the credential store
            client_cert: The client certificate to use for authorization, as a plain string
            client_key: The client key to use for authorization, as a plain string
            client_pkcs12_file: The pkcs12 file contents to use for authorization, as a plain string
                (alternative to using client_cert and client_key)
            passphrase: The passphrase to use to decrypt client_pkcs12_file or client_key file
            server_ca_cert: The server certificate, so the credential store can be trusted
            comment: An optional comment to store alongside the credential store
        """

        if not credential_store_id:
            raise RequiredArgument(
                function=cls.verify_credential_store.__name__,
                argument="credential_store_id",
            )

        cmd = XmlCommand("modify_credential_store")
        cmd.set_attribute("credential_store_id", str(credential_store_id))

        if active is not None:
            cmd.add_element("active", to_bool(active))
        if host:
            cmd.add_element("host", host)
        if port:
            cmd.add_element("port", str(port))
        if path:
            cmd.add_element("path", path)
        if comment:
            cmd.add_element("comment", comment)

        preferences = cmd.add_element("preferences")

        if app_id:
            preferences.add_element("app_id", app_id)
        if client_cert:
            preferences.add_element(
                "client_cert",
                b64encode(client_cert.encode("ascii")).decode("ascii"),
            )
        if client_key:
            preferences.add_element(
                "client_key",
                b64encode(client_key.encode("ascii")).decode("ascii"),
            )
        if client_pkcs12_file:
            preferences.add_element(
                "client_pkcs12_file",
                b64encode(client_pkcs12_file.encode("ascii")).decode("ascii"),
            )
        if passphrase:
            preferences.add_element("passphrase", passphrase)
        if server_ca_cert:
            preferences.add_element(
                "server_ca_cert",
                b64encode(server_ca_cert.encode("ascii")).decode("ascii"),
            )

        return cmd

    @classmethod
    def verify_credential_store(
        cls,
        credential_store_id: EntityID,
    ) -> Request:
        """Verify that the connection to a credential store works

        Args:
            credential_store_id: The uuid of the credential store to verify
        """
        if not credential_store_id:
            raise RequiredArgument(
                function=cls.verify_credential_store.__name__,
                argument="credential_store_id",
            )

        cmd = XmlCommand("verify_credential_store")
        cmd.set_attribute("credential_store_id", str(credential_store_id))
        return cmd



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.protocols.gmp import GMPv227

from .. import GmpTestCase as BaseGMPTestCase


class GMPTestCase(BaseGMPTestCase):
    gmp_class = GMPv227



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Optional

from gvm._enum import Enum


class EntityType(Enum):
    """Enum for entity types"""

    ALERT = "alert"
    ASSET = "asset"
    AUDIT = "audit"
    CERT_BUND_ADV = "cert_bund_adv"
    CPE = "cpe"
    CREDENTIAL = "credential"
    CVE = "cve"
    DFN_CERT_ADV = "dfn_cert_adv"
    FILTER = "filter"
    GROUP = "group"
    HOST = "host"
    INFO = "info"
    NOTE = "note"
    NVT = "nvt"
    OPERATING_SYSTEM = "os"
    OVALDEF = "ovaldef"
    OVERRIDE = "override"
    PERMISSION = "permission"
    POLICY = "policy"
    PORT_LIST = "port_list"
    REPORT = "report"
    REPORT_FORMAT = "report_format"
    RESULT = "result"
    ROLE = "role"
    SCAN_CONFIG = "config"
    SCANNER = "scanner"
    SCHEDULE = "schedule"
    TAG = "tag"
    TARGET = "target"
    TASK = "task"
    TICKET = "ticket"
    TLS_CERTIFICATE = "tls_certificate"
    USER = "user"
    VULNERABILITY = "vuln"

    @classmethod
    def from_string(
        cls,
        entity_type: str | None,
    ) -> Optional["EntityType"]:
        """Convert a entity type string to an actual EntityType instance

        Arguments:
            entity_type: Entity type string to convert to a EntityType
        """
        if entity_type == "vuln":
            return cls.VULNERABILITY

        if entity_type == "os":
            return cls.OPERATING_SYSTEM

        if entity_type == "config":
            return cls.SCAN_CONFIG

        return super().from_string(entity_type)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import GvmError
from gvm.protocols.core import Connection, InvalidStateError


class RequestMock:
    def __init__(self, data: bytes) -> None:
        self._data = data

    def __bytes__(self) -> bytes:
        return self._data


class ConnectionTestCase(unittest.TestCase):
    def test_send_request(self) -> None:
        request = RequestMock(b"<request/>")
        connection = Connection()

        data = connection.send(request)
        self.assertEqual(data, b"<request/>")

    def test_receive_data(self) -> None:
        request = RequestMock(b"<request/>")
        connection = Connection()

        connection.send(request)
        response = connection.receive_data(b"<response")

        self.assertIsNone(response)
        response = connection.receive_data(b' status="200"/>')

        self.assertIsNotNone(response)
        self.assertTrue(response.is_success)  # type: ignore

        another_request = RequestMock(b"<another_request/>")
        connection.send(another_request)

        response = connection.receive_data(b"<another_response")

        self.assertIsNone(response)
        response = connection.receive_data(b' status="200"/>')

        self.assertIsNotNone(response)
        self.assertTrue(response.is_success)  # type: ignore

    def test_receive_invalid_data(self) -> None:
        request = RequestMock(b"<request/>")
        connection = Connection()

        connection.send(request)
        with self.assertRaisesRegex(
            GvmError,
            "^Cannot parse XML response. Response data read b'<response<>'$",
        ):
            connection.receive_data(b"<response<>")

    def test_error_state_close(self) -> None:
        request = RequestMock(b"<request/>")
        connection = Connection()

        connection.send(request)
        with self.assertRaisesRegex(
            GvmError,
            "^Cannot parse XML response. Response data read b'<response<>'$",
        ):
            connection.receive_data(b"<response<>")

        with self.assertRaisesRegex(
            InvalidStateError,
            "^The connection is in an error state. Please close the connection.$",
        ):
            connection.receive_data(b"<response/>")

        with self.assertRaisesRegex(
            InvalidStateError,
            "^The connection is in an error state. Please close the connection.$",
        ):
            connection.send(request)

        connection.close()
        connection.send(request)

    def test_send_when_receiving_data(self) -> None:
        request = RequestMock(b"<request/>")
        connection = Connection()

        connection.send(request)
        connection.receive_data(b"<response")

        with self.assertRaisesRegex(
            InvalidStateError,
            "^Invalid State$",
        ):
            connection.send(request)

    def test_close_before_send(self) -> None:
        request = RequestMock(b"<request/>")
        connection = Connection()

        connection.close()

        data = connection.send(request)
        self.assertEqual(data, b"<request/>")

    def test_close_after_send(self) -> None:
        request = RequestMock(b"<request/>")
        connection = Connection()

        data = connection.send(request)
        self.assertEqual(data, b"<request/>")

        connection.close()

        with self.assertRaisesRegex(
            InvalidStateError,
            "^Invalid State$",
        ):
            connection.receive_data(b"<response")

        data = connection.send(request)
        self.assertEqual(data, b"<request/>")

    def test_close_after_receive_data(self) -> None:
        request = RequestMock(b"<request/>")
        connection = Connection()

        data = connection.send(request)
        self.assertEqual(data, b"<request/>")

        connection.receive_data(b"<response")

        connection.close()

        with self.assertRaisesRegex(
            InvalidStateError,
            "^Invalid State$",
        ):
            connection.receive_data(b' status="200"/>')

        data = connection.send(request)
        self.assertEqual(data, b"<request/>")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm._enum import Enum
from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class PortRangeType(Enum):
    """Enum for port range type"""

    TCP = "TCP"
    UDP = "UDP"


class PortLists:
    @classmethod
    def clone_port_list(cls, port_list_id: EntityID) -> Request:
        """Clone an existing port list

        Args:
            port_list_id: UUID of an existing port list to clone from
        """
        if not port_list_id:
            raise RequiredArgument(
                function=cls.clone_port_list.__name__, argument="port_list_id"
            )

        cmd = XmlCommand("create_port_list")
        cmd.add_element("copy", str(port_list_id))
        return cmd

    @classmethod
    def create_port_list(
        cls, name: str, port_range: str, *, comment: str | None = None
    ) -> Request:
        """Create a new port list

        Args:
            name: Name of the new port list
            port_range: Port list ranges e.g. `"T: 1-1234"` for tcp port
                1 - 1234
            comment: Comment for the port list
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_port_list.__name__, argument="name"
            )

        if not port_range:
            raise RequiredArgument(
                function=cls.create_port_list.__name__, argument="port_range"
            )

        cmd = XmlCommand("create_port_list")
        cmd.add_element("name", name)
        cmd.add_element("port_range", port_range)

        if comment:
            cmd.add_element("comment", comment)

        return cmd

    @classmethod
    def create_port_range(
        cls,
        port_list_id: EntityID,
        start: int,
        end: int,
        port_range_type: str | PortRangeType,
        *,
        comment: str | None = None,
    ) -> Request:
        """Create new port range

        Args:
            port_list_id: UUID of the port list to which to add the range
            start: The first port in the range
            end: The last port in the range
            port_range_type: The type of the ports: TCP, UDP, ...
            comment: Comment for the port range
        """
        if not port_list_id:
            raise RequiredArgument(
                function=cls.create_port_range.__name__,
                argument="port_list_id",
            )

        if not port_range_type:
            raise RequiredArgument(
                function=cls.create_port_range.__name__,
                argument="port_range_type",
            )

        if not start:
            raise RequiredArgument(
                function=cls.create_port_range.__name__, argument="start"
            )

        if not end:
            raise RequiredArgument(
                function=cls.create_port_range.__name__, argument="end"
            )

        if not isinstance(port_range_type, PortRangeType):
            port_range_type = PortRangeType(port_range_type)

        cmd = XmlCommand("create_port_range")
        cmd.add_element("port_list", attrs={"id": str(port_list_id)})
        cmd.add_element("start", str(start))
        cmd.add_element("end", str(end))
        cmd.add_element("type", port_range_type.value)

        if comment:
            cmd.add_element("comment", comment)

        return cmd

    @classmethod
    def delete_port_list(
        cls, port_list_id: EntityID, *, ultimate: bool = False
    ) -> Request:
        """Deletes an existing port list

        Args:
            port_list_id: UUID of the port list to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not port_list_id:
            raise RequiredArgument(
                function=cls.delete_port_list.__name__, argument="port_list_id"
            )

        cmd = XmlCommand("delete_port_list")
        cmd.set_attribute("port_list_id", str(port_list_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @classmethod
    def delete_port_range(cls, port_range_id: EntityID) -> Request:
        """Deletes an existing port range

        Args:
            port_range_id: UUID of the port range to be deleted.
        """
        if not port_range_id:
            raise RequiredArgument(
                function=cls.delete_port_range.__name__,
                argument="port_range_id",
            )

        cmd = XmlCommand("delete_port_range")
        cmd.set_attribute("port_range_id", str(port_range_id))

        return cmd

    @classmethod
    def get_port_lists(
        cls,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        details: bool | None = None,
        targets: bool | None = None,
        trash: bool | None = None,
    ) -> Request:
        """Request a list of port lists

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            details: Whether to include full port list details
            targets: Whether to include targets using this port list
            trash: Whether to get port lists in the trashcan instead
        """
        cmd = XmlCommand("get_port_lists")

        cmd.add_filter(filter_string, filter_id)

        if details is not None:
            cmd.set_attribute("details", to_bool(details))

        if targets is not None:
            cmd.set_attribute("targets", to_bool(targets))

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        return cmd

    @classmethod
    def get_port_list(cls, port_list_id: EntityID) -> Request:
        """Request a single port list

        Args:
            port_list_id: UUID of an existing port list
        """
        cmd = XmlCommand("get_port_lists")

        if not port_list_id:
            raise RequiredArgument(
                function=cls.get_port_list.__name__, argument="port_list_id"
            )

        cmd.set_attribute("port_list_id", str(port_list_id))

        # for single entity always request all details

        cmd.set_attribute("details", "1")
        return cmd

    @classmethod
    def modify_port_list(
        cls,
        port_list_id: EntityID,
        *,
        comment: str | None = None,
        name: str | None = None,
    ) -> Request:
        """Modifies an existing port list.

        Args:
            port_list_id: UUID of port list to modify.
            name: Name of port list.
            comment: Comment on port list.
        """
        if not port_list_id:
            raise RequiredArgument(
                function=cls.modify_port_list.__name__, argument="port_list_id"
            )
        cmd = XmlCommand("modify_port_list")
        cmd.set_attribute("port_list_id", str(port_list_id))

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        return cmd



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.tags import (
    GmpCloneTagTestMixin,
    GmpCreateTagTestMixin,
    GmpDeleteTagTestMixin,
    GmpGetTagsTestMixin,
    GmpGetTagTestMixin,
    GmpModifyTagTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225DeleteTagTestCase(GmpDeleteTagTestMixin, GMPTestCase):
    pass


class Gmpv225GetTagTestCase(GmpGetTagTestMixin, GMPTestCase):
    pass


class Gmpv225GetTagsTestCase(GmpGetTagsTestMixin, GMPTestCase):
    pass


class Gmpv225CloneTagTestCase(GmpCloneTagTestMixin, GMPTestCase):
    pass


class Gmpv225CreateTagTestCase(GmpCreateTagTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyTagTestCase(GmpModifyTagTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .credentials import (
    GmpCloneCredentialTestMixin,
    GmpCreateCredentialTestMixin,
    GmpDeleteCredentialTestMixin,
    GmpGetCredentialsTestMixin,
    GmpGetCredentialTestMixin,
    GmpModifyCredentialTestMixin,
)


class Gmpv224CloneCredentialTestCase(
    GmpCloneCredentialTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224CreateCredentialTestCase(
    GmpCreateCredentialTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224DeleteCredentialTestCase(
    GmpDeleteCredentialTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224GetCredentialTestCase(GmpGetCredentialTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetCredentialsTestCase(
    GmpGetCredentialsTestMixin, Gmpv224TestCase
):
    pass


class Gmpv224ModifyCredentialTestCase(
    GmpModifyCredentialTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetProtocolVersionTestCase:
    def test_protocol_version(self):
        self.assertEqual(self.gmp.get_protocol_version(), (22, 4))



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v226 import EntityType


class GetEntityTypeFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            EntityType.from_string("foo")

    def test_none_or_empty(self):
        ct = EntityType.from_string(None)
        self.assertIsNone(ct)
        ct = EntityType.from_string("")
        self.assertIsNone(ct)

    def test_audit(self):
        ct = EntityType.from_string("audit")
        self.assertEqual(ct, EntityType.AUDIT)

    def test_alert(self):
        ct = EntityType.from_string("alert")
        self.assertEqual(ct, EntityType.ALERT)

    def test_asset(self):
        ct = EntityType.from_string("asset")
        self.assertEqual(ct, EntityType.ASSET)

    def test_cert_bund_adv(self):
        ct = EntityType.from_string("cert_bund_adv")
        self.assertEqual(ct, EntityType.CERT_BUND_ADV)

    def test_cpe(self):
        ct = EntityType.from_string("cpe")
        self.assertEqual(ct, EntityType.CPE)

    def test_credential(self):
        ct = EntityType.from_string("credential")
        self.assertEqual(ct, EntityType.CREDENTIAL)

    def test_dfn_cert_adv(self):
        ct = EntityType.from_string("dfn_cert_adv")
        self.assertEqual(ct, EntityType.DFN_CERT_ADV)

    def test_filter(self):
        ct = EntityType.from_string("filter")
        self.assertEqual(ct, EntityType.FILTER)

    def test_group(self):
        ct = EntityType.from_string("group")
        self.assertEqual(ct, EntityType.GROUP)

    def test_host(self):
        ct = EntityType.from_string("host")
        self.assertEqual(ct, EntityType.HOST)

    def test_info(self):
        ct = EntityType.from_string("info")
        self.assertEqual(ct, EntityType.INFO)

    def test_note(self):
        ct = EntityType.from_string("note")
        self.assertEqual(ct, EntityType.NOTE)

    def test_nvt(self):
        ct = EntityType.from_string("nvt")
        self.assertEqual(ct, EntityType.NVT)

    def test_operating_system(self):
        ct = EntityType.from_string("os")
        self.assertEqual(ct, EntityType.OPERATING_SYSTEM)

        ct = EntityType.from_string("operating_system")
        self.assertEqual(ct, EntityType.OPERATING_SYSTEM)

    def test_ovaldef(self):
        ct = EntityType.from_string("ovaldef")
        self.assertEqual(ct, EntityType.OVALDEF)

    def test_override(self):
        ct = EntityType.from_string("override")
        self.assertEqual(ct, EntityType.OVERRIDE)

    def test_permission(self):
        ct = EntityType.from_string("permission")
        self.assertEqual(ct, EntityType.PERMISSION)

    def test_policy(self):
        ct = EntityType.from_string("policy")
        self.assertEqual(ct, EntityType.POLICY)

    def test_port_list(self):
        ct = EntityType.from_string("port_list")
        self.assertEqual(ct, EntityType.PORT_LIST)

    def test_report(self):
        ct = EntityType.from_string("report")
        self.assertEqual(ct, EntityType.REPORT)

    def test_report_format(self):
        ct = EntityType.from_string("report_format")
        self.assertEqual(ct, EntityType.REPORT_FORMAT)

    def test_result(self):
        ct = EntityType.from_string("result")
        self.assertEqual(ct, EntityType.RESULT)

    def test_role(self):
        ct = EntityType.from_string("role")
        self.assertEqual(ct, EntityType.ROLE)

    def test_scan_config(self):
        ct = EntityType.from_string("config")
        self.assertEqual(ct, EntityType.SCAN_CONFIG)

        ct = EntityType.from_string("scan_config")
        self.assertEqual(ct, EntityType.SCAN_CONFIG)

    def test_scanner(self):
        ct = EntityType.from_string("scanner")
        self.assertEqual(ct, EntityType.SCANNER)

    def test_schedule(self):
        ct = EntityType.from_string("schedule")
        self.assertEqual(ct, EntityType.SCHEDULE)

    def test_tag(self):
        ct = EntityType.from_string("tag")
        self.assertEqual(ct, EntityType.TAG)

    def test_target(self):
        ct = EntityType.from_string("target")
        self.assertEqual(ct, EntityType.TARGET)

    def test_task(self):
        ct = EntityType.from_string("task")
        self.assertEqual(ct, EntityType.TASK)

    def test_ticket(self):
        ct = EntityType.from_string("ticket")
        self.assertEqual(ct, EntityType.TICKET)

    def test_tls_certificate(self):
        ft = EntityType.from_string("tls_certificate")
        self.assertEqual(ft, EntityType.TLS_CERTIFICATE)

    def test_user(self):
        ct = EntityType.from_string("user")
        self.assertEqual(ct, EntityType.USER)

    def test_vulnerability(self):
        ct = EntityType.from_string("vuln")
        self.assertEqual(ct, EntityType.VULNERABILITY)

        ct = EntityType.from_string("vulnerability")
        self.assertEqual(ct, EntityType.VULNERABILITY)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_get_result import GmpGetResultTestMixin
from .test_get_results import GmpGetResultsTestMixin

__all__ = (
    "GmpGetResultTestMixin",
    "GmpGetResultsTestMixin",
)



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpCloneTicketTestMixin:
    def test_clone(self):
        self.gmp.clone_ticket("t1")

        self.connection.send.has_been_called_with(
            b"<create_ticket><copy>t1</copy></create_ticket>"
        )

        self.gmp.clone_ticket(ticket_id="t1")

        self.connection.send.has_been_called_with(
            b"<create_ticket><copy>t1</copy></create_ticket>"
        )

    def test_missing_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.clone_ticket("")

        with self.assertRaises(RequiredArgument):
            self.gmp.clone_ticket(None)

        with self.assertRaises(RequiredArgument):
            self.gmp.clone_ticket(ticket_id=None)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import FilterType


class GmpCreateFilterTestMixin:
    def test_all_available_filters_types_correct(self):
        for filter_type in list(FilterType):
            self.gmp.create_filter(
                name="f1",
                term="sort-reverse=threat first=1 rows=1000",
                filter_type=filter_type,
            )

            self.connection.send.has_been_called_with(
                "<create_filter>"
                "<name>f1</name>"
                "<term>sort-reverse=threat first=1 rows=1000</term>"
                f"<type>{filter_type.value}</type>"
                "</create_filter>".encode()
            )

    def test_create_filter_invalid_filter_type(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.create_filter(
                name="f1",
                term="sort-reverse=threat result_hosts_only=1 "
                "notes=1 overrides=1 levels=hml first=1 rows=1000",
                filter_type="foo",
            )

    def test_create_filter_no_filter_type(self):
        self.gmp.create_filter(
            name="f1",
            term="sort-reverse=threat result_hosts_only=1 "
            "notes=1 overrides=1 levels=hml first=1 rows=1000",
            comment="foo",
        )

        self.connection.send.has_been_called_with(
            b"<create_filter>"
            b"<name>f1</name>"
            b"<comment>foo</comment>"
            b"<term>sort-reverse=threat result_hosts_only=1 notes=1 "
            b"overrides=1 levels=hml first=1 rows=1000</term>"
            b"</create_filter>"
        )

    def test_create_filter(self):
        self.gmp.create_filter(
            name="f1",
            term="sort-reverse=threat result_hosts_only=1 "
            "notes=1 overrides=1 levels=hml first=1 rows=1000",
            filter_type=FilterType.TASK,
            comment="foo",
        )

        self.connection.send.has_been_called_with(
            b"<create_filter>"
            b"<name>f1</name>"
            b"<comment>foo</comment>"
            b"<term>sort-reverse=threat result_hosts_only=1 notes=1 "
            b"overrides=1 levels=hml first=1 rows=1000</term>"
            b"<type>task</type>"
            b"</create_filter>"
        )

    def test_create_filter_without_term(self):
        self.gmp.create_filter(
            name="f1",
            filter_type=FilterType.TASK,
            comment="foo",
        )

        self.connection.send.has_been_called_with(
            b"<create_filter>"
            b"<name>f1</name>"
            b"<comment>foo</comment>"
            b"<type>task</type>"
            b"</create_filter>"
        )

    def test_create_filter_make_unique(self):
        with self.assertRaises(TypeError):
            self.gmp.create_filter(
                name="f1",
                term="sort-reverse=threat result_hosts_only=1 "
                "notes=1 overrides=1 levels=hml first=1 rows=1000",
                filter_type=FilterType.TASK,
                make_unique=True,
                comment="foo",
            )

    def test_create_filter_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_filter("")

        with self.assertRaises(RequiredArgument):
            self.gmp.create_filter(None)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import EntityType, PermissionSubjectType


class GmpModifyPermissionTestMixin:
    def test_modify_permission(self):
        self.gmp.modify_permission(permission_id="p1")

        self.connection.send.has_been_called_with(
            b'<modify_permission permission_id="p1"/>'
        )

    def test_modify_permission_missing_permission_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(permission_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(permission_id="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission("")

    def test_modify_permission_with_comment(self):
        self.gmp.modify_permission(permission_id="p1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_permission permission_id="p1">'
            b"<comment>foo</comment>"
            b"</modify_permission>"
        )

    def test_modify_permission_with_resource_id_and_type(self):
        self.gmp.modify_permission(
            permission_id="p1", resource_id="r1", resource_type=EntityType.TASK
        )

        self.connection.send.has_been_called_with(
            b'<modify_permission permission_id="p1">'
            b'<resource id="r1">'
            b"<type>task</type>"
            b"</resource>"
            b"</modify_permission>"
        )

    def test_modify_permission_with_resource_id_and_type_audit(self):
        self.gmp.modify_permission(
            permission_id="p1", resource_id="r1", resource_type=EntityType.AUDIT
        )

        self.connection.send.has_been_called_with(
            b'<modify_permission permission_id="p1">'
            b'<resource id="r1">'
            b"<type>task</type>"
            b"</resource>"
            b"</modify_permission>"
        )

    def test_modify_permission_with_resource_id_and_type_policy(self):
        self.gmp.modify_permission(
            permission_id="p1",
            resource_id="r1",
            resource_type=EntityType.POLICY,
        )

        self.connection.send.has_been_called_with(
            b'<modify_permission permission_id="p1">'
            b'<resource id="r1">'
            b"<type>config</type>"
            b"</resource>"
            b"</modify_permission>"
        )

    def test_modify_permission_with_missing_resource_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(
                permission_id="p1",
                resource_id="",
                resource_type=EntityType.TASK,
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(
                permission_id="p1", resource_type=EntityType.TASK
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(
                permission_id="p1",
                resource_id=None,
                resource_type=EntityType.TASK,
            )

    def test_modify_permission_with_missing_resource_type(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(
                permission_id="p1", resource_id="r1", resource_type=""
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(permission_id="p1", resource_id="r1")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(
                permission_id="p1", resource_id="r1", resource_type=None
            )

    def test_modify_permission_with_invalid_resource_type(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_permission(
                permission_id="p1", resource_id="r1", resource_type="blah"
            )

    def test_modify_permission_with_subject_id_and_type(self):
        self.gmp.modify_permission(
            permission_id="p1",
            subject_id="s1",
            subject_type=PermissionSubjectType.ROLE,
        )

        self.connection.send.has_been_called_with(
            b'<modify_permission permission_id="p1">'
            b'<subject id="s1">'
            b"<type>role</type>"
            b"</subject>"
            b"</modify_permission>"
        )

        self.gmp.modify_permission(
            permission_id="p1",
            subject_id="s1",
            subject_type=PermissionSubjectType.USER,
        )

        self.connection.send.has_been_called_with(
            b'<modify_permission permission_id="p1">'
            b'<subject id="s1">'
            b"<type>user</type>"
            b"</subject>"
            b"</modify_permission>"
        )

        self.gmp.modify_permission(
            permission_id="p1",
            subject_id="s1",
            subject_type=PermissionSubjectType.GROUP,
        )

        self.connection.send.has_been_called_with(
            b'<modify_permission permission_id="p1">'
            b'<subject id="s1">'
            b"<type>group</type>"
            b"</subject>"
            b"</modify_permission>"
        )

    def test_modify_permission_missing_subject_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(
                permission_id="p1", subject_type=PermissionSubjectType.ROLE
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(
                permission_id="p1",
                subject_type=PermissionSubjectType.ROLE,
                subject_id="",
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(
                permission_id="p1",
                subject_type=PermissionSubjectType.ROLE,
                subject_id=None,
            )

    def test_modify_permission_invalid_subject_type(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_permission(
                permission_id="p1", subject_id="s1", subject_type="foo"
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(
                permission_id="p1", subject_id="s1", subject_type=""
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_permission(
                permission_id="p1", subject_id="s1", subject_type=None
            )

    def test_modify_permission_with_name(self):
        self.gmp.modify_permission(permission_id="p1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_permission permission_id="p1">'
            b"<name>foo</name>"
            b"</modify_permission>"
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.filters import (
    GmpCloneFilterTestMixin,
    GmpCreateFilterTestMixin,
    GmpDeleteFilterTestMixin,
    GmpGetFiltersTestMixin,
    GmpGetFilterTestMixin,
    GmpModifyFilterTestMixin,
)
from ...gmpv226 import GMPTestCase


class GMPDeleteFilterTestCase(GmpDeleteFilterTestMixin, GMPTestCase):
    pass


class GMPGetFilterTestCase(GmpGetFilterTestMixin, GMPTestCase):
    pass


class GMPGetFiltersTestCase(GmpGetFiltersTestMixin, GMPTestCase):
    pass


class GMPCloneFilterTestCase(GmpCloneFilterTestMixin, GMPTestCase):
    pass


class GMPCreateFilterTestCase(GmpCreateFilterTestMixin, GMPTestCase):
    pass


class GMPModifyFilterTestCase(GmpModifyFilterTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetOperatingSystemsTestMixin:
    def test_get_operating_systems(self):
        self.gmp.get_operating_systems()

        self.connection.send.has_been_called_with(b'<get_assets type="os"/>')

    def test_get_operating_systems_details(self):
        self.gmp.get_operating_systems(details=True)

        self.connection.send.has_been_called_with(
            b'<get_assets type="os" details="1"/>'
        )

        self.gmp.get_operating_systems(details=False)

        self.connection.send.has_been_called_with(
            b'<get_assets type="os" details="0"/>'
        )

    def test_get_operating_systems_with_filter_string(self):
        self.gmp.get_operating_systems(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_assets type="os" filter="foo=bar"/>'
        )

    def test_get_operating_systems_with_filter_id(self):
        self.gmp.get_operating_systems(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_assets type="os" filt_id="f1"/>'
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.vulnerabilities import (
    GmpGetVulnerabilitiesTestMixin,
    GmpGetVulnerabilityTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225GetVulnerabilityTestCase(
    GmpGetVulnerabilityTestMixin, GMPTestCase
):
    pass


class Gmpv225GetVulnerabilitiesTestCase(
    GmpGetVulnerabilitiesTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from decimal import Decimal

from gvm.errors import InvalidArgument, RequiredArgument


class GmpCreateNoteTestMixin:
    def test_create_note(self):
        self.gmp.create_note("foo", nvt_oid="oid1")

        self.connection.send.has_been_called_with(
            b'<create_note><text>foo</text><nvt oid="oid1"/></create_note>'
        )

    def test_create_note_missing_text(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_note(None, "od1")

        with self.assertRaises(RequiredArgument):
            self.gmp.create_note("", "oid1")

    def test_create_note_missing_nvt_oid(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_note("foo", None)

        with self.assertRaises(RequiredArgument):
            self.gmp.create_note("foo", "")

    def test_create_note_with_hosts(self):
        self.gmp.create_note("foo", nvt_oid="oid1", hosts=[])

        self.connection.send.has_been_called_with(
            b'<create_note><text>foo</text><nvt oid="oid1"/></create_note>'
        )

        self.gmp.create_note("foo", nvt_oid="oid1", hosts=["h1", "h2"])

        self.connection.send.has_been_called_with(
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<hosts>h1,h2</hosts>"
            b"</create_note>"
        )

    def test_create_note_with_port(self):
        self.gmp.create_note("foo", nvt_oid="oid1", port="666/tcp")

        self.connection.send.has_been_called_with(
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<port>666/tcp</port>"
            b"</create_note>"
        )

    def test_create_note_with_result_id(self):
        self.gmp.create_note("foo", nvt_oid="oid1", result_id="r1")

        self.connection.send.has_been_called_with(
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b'<result id="r1"/>'
            b"</create_note>"
        )

    def test_create_note_with_task_id(self):
        self.gmp.create_note("foo", nvt_oid="oid1", task_id="t1")

        self.connection.send.has_been_called_with(
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b'<task id="t1"/>'
            b"</create_note>"
        )

    def test_create_note_with_severity(self):
        self.gmp.create_note("foo", nvt_oid="oid1", severity="5.5")

        self.connection.send.has_been_called_with(
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<severity>5.5</severity>"
            b"</create_note>"
        )

        self.gmp.create_note("foo", nvt_oid="oid1", severity=5.5)

        self.connection.send.has_been_called_with(
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<severity>5.5</severity>"
            b"</create_note>"
        )

        self.gmp.create_note("foo", nvt_oid="oid1", severity=Decimal("5.5"))

        self.connection.send.has_been_called_with(
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<severity>5.5</severity>"
            b"</create_note>"
        )

    def test_create_note_with_days_active(self):
        self.gmp.create_note("foo", nvt_oid="oid1", days_active=0)

        self.connection.send.has_been_called_with(
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<active>0</active>"
            b"</create_note>"
        )

        self.gmp.create_note("foo", nvt_oid="oid1", days_active=-1)

        self.connection.send.has_been_called_with(
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<active>-1</active>"
            b"</create_note>"
        )

        self.gmp.create_note("foo", nvt_oid="oid1", days_active=3600)

        self.connection.send.has_been_called_with(
            b"<create_note>"
            b"<text>foo</text>"
            b'<nvt oid="oid1"/>'
            b"<active>3600</active>"
            b"</create_note>"
        )

    def test_create_note_with_invalid_port(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.create_note(text="foo", nvt_oid="oid1", port="123")

        with self.assertRaises(InvalidArgument):
            self.gmp.create_note(text="foo", nvt_oid="oid1", port="tcp/123")



# SPDX-FileCopyrightText: 2020-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from decimal import Decimal

from gvm.errors import InvalidArgument, RequiredArgument


class GmpModifyOverrideTestMixin:
    def test_modify_override(self):
        self.gmp.modify_override(override_id="o1", text="foo")

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"</modify_override>"
        )

    def test_modify_override_missing_override_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_override(override_id=None, text="foo")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_override(override_id="", text="foo")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_override("", text="foo")

    def test_modify_override_missing_text(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_override(override_id="o1", text="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_override(override_id="o1", text=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_override("o1", "")

    def test_modify_override_with_days_active(self):
        self.gmp.modify_override(override_id="o1", text="foo", days_active=0)

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<active>0</active>"
            b"</modify_override>"
        )

        self.gmp.modify_override(override_id="o1", text="foo", days_active=-1)

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<active>-1</active>"
            b"</modify_override>"
        )

        self.gmp.modify_override(override_id="o1", text="foo", days_active=600)

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<active>600</active>"
            b"</modify_override>"
        )

    def test_modify_override_with_port(self):
        self.gmp.modify_override(override_id="o1", text="foo", port="123/udp")

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<port>123/udp</port>"
            b"</modify_override>"
        )

    def test_modify_override_with_hosts(self):
        self.gmp.modify_override(override_id="o1", text="foo", hosts=["foo"])

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<hosts>foo</hosts>"
            b"</modify_override>"
        )

        self.gmp.modify_override(
            override_id="o1", text="foo", hosts=["foo", "bar"]
        )

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<hosts>foo,bar</hosts>"
            b"</modify_override>"
        )

    def test_modify_override_with_result_id(self):
        self.gmp.modify_override(override_id="o1", text="foo", result_id="r1")

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b'<result id="r1"/>'
            b"</modify_override>"
        )

    def test_modify_override_with_task_id(self):
        self.gmp.modify_override(override_id="o1", text="foo", task_id="r1")

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b'<task id="r1"/>'
            b"</modify_override>"
        )

    def test_modify_override_with_severity(self):
        self.gmp.modify_override(override_id="o1", text="foo", severity="5.5")

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_override>"
        )

        self.gmp.modify_override(override_id="o1", text="foo", severity=5.5)

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_override>"
        )

        self.gmp.modify_override(
            override_id="o1", text="foo", severity=Decimal("5.5")
        )

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<severity>5.5</severity>"
            b"</modify_override>"
        )

    def test_modify_override_with_new_severity(self):
        self.gmp.modify_override(
            override_id="o1", text="foo", new_severity="5.5"
        )

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<new_severity>5.5</new_severity>"
            b"</modify_override>"
        )

        self.gmp.modify_override(override_id="o1", text="foo", new_severity=5.5)

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<new_severity>5.5</new_severity>"
            b"</modify_override>"
        )

        self.gmp.modify_override(
            override_id="o1", text="foo", new_severity=Decimal("5.5")
        )

        self.connection.send.has_been_called_with(
            b'<modify_override override_id="o1">'
            b"<text>foo</text>"
            b"<new_severity>5.5</new_severity>"
            b"</modify_override>"
        )

    def test_modify_override_with_invalid_port(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_override(override_id="o1", text="foo", port="123")



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.tickets import (
    GmpCloneTicketTestMixin,
    GmpCreateTicketTestMixin,
    GmpDeleteTicketTestMixin,
    GmpGetTicketsTestMixin,
    GmpGetTicketTestMixin,
    GmpModifyTicketTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225DeleteTicketTestCase(GmpDeleteTicketTestMixin, GMPTestCase):
    pass


class Gmpv225GetTicketTestCase(GmpGetTicketTestMixin, GMPTestCase):
    pass


class Gmpv225GetTicketsTestCase(GmpGetTicketsTestMixin, GMPTestCase):
    pass


class Gmpv225CloneTicketTestCase(GmpCloneTicketTestMixin, GMPTestCase):
    pass


class Gmpv225CreateTicketTestCase(GmpCreateTicketTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyTicketTestCase(GmpModifyTicketTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_tls_certificate import GmpCloneTLSCertificateTestMixin
from .test_create_tls_certificate import GmpCreateTLSCertificateTestMixin
from .test_delete_tls_certificate import GmpDeleteTLSCertificateTestMixin
from .test_get_tls_certificate import GmpGetTLSCertificateTestMixin
from .test_get_tls_certificates import GmpGetTLSCertificatesTestMixin
from .test_modify_tls_certificate import GmpModifyTLSCertificateTestMixin

__all__ = (
    "GmpCloneTLSCertificateTestMixin",
    "GmpCreateTLSCertificateTestMixin",
    "GmpDeleteTLSCertificateTestMixin",
    "GmpGetTLSCertificateTestMixin",
    "GmpGetTLSCertificatesTestMixin",
    "GmpModifyTLSCertificateTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from collections import OrderedDict

from gvm.errors import RequiredArgument


class GmpModifyAuthTestMixin:
    def test_modify_auth(self):
        self.gmp.modify_auth(
            "foo", OrderedDict([("foo", "bar"), ("lorem", "ipsum")])
        )

        self.connection.send.has_been_called_with(
            b"<modify_auth>"
            b'<group name="foo">'
            b"<auth_conf_setting>"
            b"<key>foo</key>"
            b"<value>bar</value>"
            b"</auth_conf_setting>"
            b"<auth_conf_setting>"
            b"<key>lorem</key>"
            b"<value>ipsum</value>"
            b"</auth_conf_setting>"
            b"</group>"
            b"</modify_auth>"
        )

    def test_modify_auth_missing_group_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_auth(
                group_name=None, auth_conf_settings={"foo": "bar"}
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_auth(
                group_name="", auth_conf_settings={"foo": "bar"}
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_auth("", auth_conf_settings={"foo": "bar"})

    def test_modify_auth_auth_conf_settings(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_auth(group_name="foo", auth_conf_settings=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_auth(group_name="foo", auth_conf_settings="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_auth("foo", "")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_auth("foo", {})



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.next import CredentialStoreCredentialType


class GmpCreateCredentialStoreCredentialTestMixin:
    def test_create_cs_up_credential_missing_name(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_credential_store_credential(
                name="",
                credential_type=CredentialStoreCredentialType.USERNAME_PASSWORD,
                vault_id="foo",
                host_identifier="bar",
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_credential_store_credential(
                name=None,
                credential_type=CredentialStoreCredentialType.USERNAME_PASSWORD,
                vault_id="foo",
                host_identifier="bar",
            )

    def test_create_cs_up_credential(self):
        self.gmp.create_credential_store_credential(
            name="foo",
            credential_type=CredentialStoreCredentialType.USERNAME_PASSWORD,
            comment="bar",
            vault_id="123",
            host_identifier="456",
        )

        self.connection.send.has_been_called_with(
            b"<create_credential>"
            b"<name>foo</name>"
            b"<type>cs_up</type>"
            b"<comment>bar</comment>"
            b"<vault_id>123</vault_id>"
            b"<host_identifier>456</host_identifier>"
            b"</create_credential>"
        )

    def test_create_cs_up_credential_with_credential_store_id(self):
        self.gmp.create_credential_store_credential(
            name="foo",
            credential_type=CredentialStoreCredentialType.USERNAME_PASSWORD,
            comment="bar",
            credential_store_id="abc",
            vault_id="123",
            host_identifier="456",
        )

        self.connection.send.has_been_called_with(
            b"<create_credential>"
            b"<name>foo</name>"
            b"<type>cs_up</type>"
            b"<comment>bar</comment>"
            b"<credential_store_id>abc</credential_store_id>"
            b"<vault_id>123</vault_id>"
            b"<host_identifier>456</host_identifier>"
            b"</create_credential>"
        )

    def test_create_cs_up_credential_with_missing_vault_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_credential_store_credential(
                name="foo",
                credential_type=CredentialStoreCredentialType.USERNAME_PASSWORD,
                comment="bar",
                host_identifier="456",
            )

    def test_create_cs_up_credential_with_missing_host_identifier(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_credential_store_credential(
                name="foo",
                credential_type=CredentialStoreCredentialType.USERNAME_PASSWORD,
                comment="bar",
                vault_id="123",
            )

    def test_create_cs_cc_credential(self):
        self.gmp.create_credential_store_credential(
            name="foo",
            credential_type=CredentialStoreCredentialType.CLIENT_CERTIFICATE,
            vault_id="123",
            host_identifier="456",
        )

        self.connection.send.has_been_called_with(
            b"<create_credential>"
            b"<name>foo</name>"
            b"<type>cs_cc</type>"
            b"<vault_id>123</vault_id>"
            b"<host_identifier>456</host_identifier>"
            b"</create_credential>"
        )

    def test_create_cs_usk_credential(self):
        self.gmp.create_credential_store_credential(
            name="foo",
            credential_type=CredentialStoreCredentialType.USERNAME_SSH_KEY,
            vault_id="123",
            host_identifier="456",
        )

        self.connection.send.has_been_called_with(
            b"<create_credential>"
            b"<name>foo</name>"
            b"<type>cs_usk</type>"
            b"<vault_id>123</vault_id>"
            b"<host_identifier>456</host_identifier>"
            b"</create_credential>"
        )

    def test_create_cs_snmp_credential(self):
        self.gmp.create_credential_store_credential(
            name="foo",
            credential_type=CredentialStoreCredentialType.SNMP,
            vault_id="123",
            host_identifier="456",
        )

        self.connection.send.has_been_called_with(
            b"<create_credential>"
            b"<name>foo</name>"
            b"<type>cs_snmp</type>"
            b"<vault_id>123</vault_id>"
            b"<host_identifier>456</host_identifier>"
            b"</create_credential>"
        )

    def test_create_cs_smime_credential(self):
        self.gmp.create_credential_store_credential(
            name="foo",
            credential_type=CredentialStoreCredentialType.SMIME_CERTIFICATE,
            vault_id="123",
            host_identifier="456",
        )

        self.connection.send.has_been_called_with(
            b"<create_credential>"
            b"<name>foo</name>"
            b"<type>cs_smime</type>"
            b"<vault_id>123</vault_id>"
            b"<host_identifier>456</host_identifier>"
            b"</create_credential>"
        )

    def test_create_cs_pgp_credential(self):
        self.gmp.create_credential_store_credential(
            name="foo",
            credential_type=CredentialStoreCredentialType.PGP_ENCRYPTION_KEY,
            vault_id="123",
            host_identifier="456",
        )

        self.connection.send.has_been_called_with(
            b"<create_credential>"
            b"<name>foo</name>"
            b"<type>cs_pgp</type>"
            b"<vault_id>123</vault_id>"
            b"<host_identifier>456</host_identifier>"
            b"</create_credential>"
        )

    def test_create_cs_credential_invalid_credential_type(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_credential_store_credential(
                name="foo", credential_type=None
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_credential_store_credential(
                name="foo", credential_type=""
            )

        with self.assertRaises(InvalidArgument):
            self.gmp.create_credential_store_credential(
                name="foo", credential_type="bar"
            )

    def test_create_cs_pw_credential(self):
        self.gmp.create_credential_store_credential(
            name="foo",
            credential_type=CredentialStoreCredentialType.PASSWORD_ONLY,
            vault_id="123",
            host_identifier="456",
        )
        self.connection.send.has_been_called_with(
            b"<create_credential>"
            b"<name>foo</name>"
            b"<type>cs_pw</type>"
            b"<vault_id>123</vault_id>"
            b"<host_identifier>456</host_identifier>"
            b"</create_credential>"
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_role import GmpCloneRoleTestMixin
from .test_create_role import GmpCreateRoleTestMixin
from .test_delete_role import GmpDeleteRoleTestMixin
from .test_get_role import GmpGetRoleTestMixin
from .test_get_roles import GmpGetRolesTestMixin
from .test_modify_role import GmpModifyRoleTestMixin

__all__ = (
    "GmpCloneRoleTestMixin",
    "GmpCreateRoleTestMixin",
    "GmpDeleteRoleTestMixin",
    "GmpGetRoleTestMixin",
    "GmpGetRolesTestMixin",
    "GmpModifyRoleTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import FeedType


class GmpGetFeedTestMixin:
    def test_get_feed(self):
        """
        Test basic get_feed calls with only resource_type except special
        cases for audit, policy, scan_config and task.
        """
        self.gmp.get_feed(FeedType.NVT)

        self.connection.send.has_been_called_with(b'<get_feeds type="NVT"/>')

        self.gmp.get_feed(FeedType.CERT)

        self.connection.send.has_been_called_with(b'<get_feeds type="CERT"/>')

        self.gmp.get_feed(FeedType.SCAP)

        self.connection.send.has_been_called_with(b'<get_feeds type="SCAP"/>')

        self.gmp.get_feed(FeedType.GVMD_DATA)

        self.connection.send.has_been_called_with(
            b'<get_feeds type="GVMD_DATA"/>'
        )

    def test_get_feed_missing_type(self):
        """
        Test get_feed calls with missing resource_type
        """
        with self.assertRaises(RequiredArgument):
            self.gmp.get_feed(feed_type=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_feed(feed_type="")

        with self.assertRaises(RequiredArgument):
            self.gmp.get_feed("")

    def test_get_feed_invalid_type(self):
        """
        Test get_feed calls with invalid resource_type
        """
        with self.assertRaises(InvalidArgument):
            self.gmp.get_feed("foo")
